@@ -7,7 +7,11 @@
 #
 #	Author:   Gregg M. Townsend
 #
-#	Date:     October 3, 1995
+#	Date:     April 9, 1996
+#
+############################################################################
+#
+#   This file is in the public domain.
 #
 ############################################################################
 #
@@ -23,11 +27,8 @@
 
 #include "icall.h"
 
-int bitcount(argc, argv)		/*: count bits in an integer */
-int argc;
-descriptor *argv;
+int bitcount(int argc, descriptor *argv) /*: count bits in an integer */
    {
-   descriptor d;
    unsigned long v;
    int n;
 
