@@ -14,10 +14,12 @@ case "$SYS" in
       ld -o $LIBNAME "$@";;
    SunOS5*)
       $CC $CFLAGS -G -K pic -o $LIBNAME "$@" -lc -lsocket;;
-   IRIX5*)
+   IRIX*)
       ld -shared -o $LIBNAME "$@";;
    OSF*)
       ld -shared -expect_unresolved '*' -o $LIBNAME "$@" -lc;;
+   Linux*)
+      gcc -shared -o $LIBNAME "$@";;
    FreeBSD*)
       ld -Bshareable -o $LIBNAME "$@" -lc;;
    *)
