@@ -41,6 +41,7 @@ struct descrip t_errorvalue;		/* tentative k_errorvalue value */
 static int list_ser;
 struct tend_desc *tend;
 
+#ifdef MSWindows
 char *getenv(char *s)
 {
 static char tmp[1537];
@@ -49,6 +50,7 @@ rv = GetEnvironmentVariable(s, tmp, 1536);
 if (rv > 0) return tmp;
 return NULL;
 }
+#endif						/* MSWindows */
 
 /*
  * An array of all characters for use in making one-character strings.
@@ -880,6 +882,8 @@ char ConsoleStringBuf[512 * 48];
 char *ConsoleStringBufPtr = ConsoleStringBuf;
 unsigned long ConsoleFlags = 0;			 /* Console flags */
 extern int ConsolePause;
+extern FILE *flog;
+
 /*
  * c_exit(i) - flush all buffers and exit with status i.
  */
@@ -900,11 +904,38 @@ int i;
       wattr(ConsoleBinding, tossanswer, strlen(tossanswer));
       waitkey(ConsoleBinding);
       }
+   if (flog) {
+      extern char *lognam;
+      extern char tmplognam[];
+      FILE *flog2;
+      int i;
+      fclose(flog);
+
+      /*
+       * try to rename, then try to copy to the permanent file name
+       */
+      i = rename(tmplognam, lognam);
+      if (i != 0) {
+         if ((flog = fopen(tmplognam, "r")) &&
+	     (flog2 = fopen(lognam, "w"))) {
+	    while ((i = getc(flog)) != EOF)
+	       putc(i, flog2);
+	    fclose(flog);
+	    fclose(flog2);
+	    remove(tmplognam);
+	    }
+         }
+
+      free(lognam);
+      }
    if (wstates != NULL) {
       PostQuitMessage(i);
       pollevent();
       }
+
+#if !MACINTOSH
 #undef exit
+#endif					/* MACINTOSH */
    exit(i);
 }
 #endif					/* ConsoleWindow */
