@@ -1076,6 +1076,9 @@ static struct il_c *db_ilc()
                            case 'g':
                               db_chstr("$cg", "oto");
                               *nxtp = new_ilc(ILC_CGto);
+#ifdef MultiThread
+#undef code
+#endif					/* MultiThead */
                               (*nxtp)->code[0] = db_ilc();
                               c = getc(db);
                               SkipWhSp(c);
