@@ -130,17 +130,14 @@
 #if COMPILER
 #undef Eve
 #undef MultiThread
-#undef Visualization
 #undef EventMon
 #endif					/* COMPILER */
 
 #ifdef Eve
 #undef EventMon
 #undef MultiThread
-#undef Visualization
 #define EventMon
 #define MultiThread
-#define Visualization
 #endif					/* Eve */
 
 #ifndef NoEnvVars
@@ -180,7 +177,7 @@
  */
 
 #ifdef EnvVars
- 
+
 #ifndef NOERRBUF
 #define NOERRBUF "NOERRBUF"
 #endif
@@ -645,7 +642,7 @@ Deliberate Syntax Error
 #ifndef SASC
 #define SASC 0
 #endif					/* SASC */
- 
+
 #ifndef WATERLOO_C_V3_0
 #define WATERLOO_C_V3_0 0
 #endif					/* WATERLOO_C_V3_0 */
@@ -673,6 +670,17 @@ Deliberate Syntax Error
 
 #endif					/* UNIX || (MACINTOSH && MPW) */
 
+#ifndef NoWildCards
+#if NT || BORLAND_286 || BORLAND_386 || MICROSOFT || SCCX_MX
+#define WildCards 1
+#else					/* NT || ... */
+#define WildCards 0
+#endif					/* NT || ... */
+#else					/* NoWildCards */
+#define WildCards 0
+#endif					/* NoWildCards */
+
+
 /*
  * End of operating-system specific code.
  */
@@ -752,7 +760,6 @@ Deliberate Syntax Error
 #define IHSize 128
 #define IHasher(x)	(((unsigned int)x)&(IHSize-1))
 
-
 #if COMPILER
 
 /*
@@ -797,11 +804,19 @@ Deliberate Syntax Error
 #define IconOptions "C:ELS:Tce:f:mn:o:p:r:stuv:x"
 #endif					/* ARM || MPW || UNIX || VMS */
 
+#if THINK_C
+#define IconOptions "C:ELS:XITce:f:mn:o:p:r:stuv:x"
+#endif
+
 #if MSDOS
 #ifdef MSWindows
 #define IconOptions "C:ELS:XITce:f:mn:o:p:qr:stuv:x"
 #else					/* MSWindows */
+#if SCCX_MX
+#define IconOptions "C:ELS:AXITce:f:mn:o:p:r:stuv:x"
+#else
 #define IconOptions "C:ELS:XITce:f:mn:o:p:r:stuv:x"
+#endif                  /* SCCX_MX */
 #endif					/* MSWindows */
 #endif					/* MSDOS */
 
@@ -840,6 +855,10 @@ Deliberate Syntax Error
 #define TUsage "[-cstuE] [-fs] [-e efile] [-o ofile]"
 #endif					/* MPW */
 
+#if THINK_C
+#define TUsage "[-cmstuE] [-fs] [-e efile] [-o ofile]"
+#endif
+
 #if OS2
 #define TUsage "[-cistuEI] [-fs] [-e efile] [-o ofile]"
 #endif					/* OS2 */
@@ -849,7 +868,11 @@ Deliberate Syntax Error
 #endif					/* MVS || VM */
 
 #if MSDOS
+#if SCCX_MX
+#define TUsage "[-cstuAEIX] [-fs] [-e efile] [-o ofile]"
+#else
 #define TUsage "[-cstuEIX] [-fs] [-e efile] [-o ofile]"
+#endif                  /* SCCX_MX */
 #endif					/* MSDOS */
 
 #endif					/* TUsage */
@@ -884,6 +907,8 @@ Deliberate Syntax Error
  *   when structures have a potentially null list of descriptors
  *   at their end.
  */
+
 #define Wsizeof(x)	((sizeof(x) + sizeof(word) - 1) / sizeof(word))
 #define Vwsizeof(x)	((sizeof(x) - sizeof(struct descrip) +\
 			   sizeof(word) - 1) / sizeof(word))
+
