@@ -94,6 +94,7 @@ FncDef(right,3)
 FncDefV(runerr)
 FncDef(seek,2)
 FncDef(seq,2)
+FncDef(serial,1)
 FncDef(set,1)
 FncDef(sort,2)
 FncDef(sortf,2)
@@ -192,14 +193,6 @@ FncDef(GetSpace,1)
 FncDef(FreeSpace,1)
 #endif					/* ArmFncs */
 
-/*
- * Functions to support program visualization.
- */
-
-#ifdef Visualization
-FncDef(serial,1)
-FncDef(structure,1)
-#endif					/* Visualization */
 
 /*
  * Event processing functions.
@@ -207,6 +200,8 @@ FncDef(structure,1)
 #ifdef EventMon
 FncDef(EvGet,2)
 FncDef(event,3)
+FncDef(eventmask,2)
+FncDef(opmask,2)
 #endif					/* EventMon */
 
 #ifdef Graphics
@@ -260,16 +255,22 @@ FncDef(WSync,1)
 FncDefV(WriteImage)
 #endif					/* Graphics */
 
+
 #ifdef MultiThread
+/*
+ * These functions are under MultiThread for no good reason.
+ */
 FncDef(cofail,1)
-FncDefV(load)
-FncDef(parent,1)
-FncDef(eventmask,2)
-FncDef(opmask,2)
 FncDef(globalnames,1)
-FncDef(keyword,2)
+FncDef(fieldnames,1)
 FncDef(localnames,2)
 FncDef(staticnames,2)
 FncDef(paramnames,2)
-FncDef(fieldnames,1)
+FncDef(structure,1)
+/*
+ * These functions are inherent to MultiThread and multiple Icon programs
+ */
+FncDefV(load)
+FncDef(parent,1)
+FncDef(keyword,2)
 #endif					/* MultiThread */
