@@ -1,6 +1,7 @@
 /*
  * graphics.h - macros and types used in Icon's graphics interface.
  */
+
 #ifdef MacGraph
 #include "::h:macgraph.h"
 #endif					/* MacGraph */
@@ -60,7 +61,7 @@
 #define FONTFLAG_NORMAL		0x00004000
 #define FONTFLAG_WIDE		0x00008000
 #define FONTFLAG_EXTENDED	0x00010000
-
+
 /*
  * EVENT HANDLING
  *
@@ -80,7 +81,6 @@
 #define EVQUESUB(w,i) *evquesub(w,i)
 #define EQUEUELEN 256
 
-
 /*
  * mode bits for the Icon window context (as opposed to X context)
  */
@@ -88,14 +88,10 @@
 #ifdef XWindows
 #define ISZOMBIE(w)     ((w)->window->bits & 1)
 #endif					/* XWindows */
-#ifdef PresentationManager
 #define ISINITIAL(w)    ((w)->window->bits & 1)
-#define ISINITIALW(w)   ((w)->bits & 1)
-#endif					/* PresentationManager */
+#define ISINITIALW(ws)   ((ws)->bits & 1)
 #define ISCURSORON(w)   ((w)->window->bits & 2)
-#ifdef PresentationManager
 #define ISCURSORONW(ws) ((ws->bits) & 2)
-#endif					/* PresentationManager */
 #define ISMAPPED(w)	((w)->window->bits & 4)
 #define ISREVERSE(w)    ((w)->context->bits & 8)
 #define ISXORREVERSE(w)	((w)->context->bits & 16)
@@ -185,7 +181,7 @@ typedef struct _wfont {
   Style     fontStyle;
   int       fontSize;
   FontInfo  fInfo;              /* I-173 */
-#endif                          /* MacGraph */
+#endif					/* MacGraph */
 #ifdef XWindows
   char	      *	name;			/* name for WAttrib and fontsearch */
   int		height;			/* font height */
@@ -206,7 +202,6 @@ typedef struct _wfont {
 #endif					/* MSWindows */
 } wfont, *wfp;
 
-
 /*
  * These structures and definitions are used for colors and images.
  */
@@ -227,12 +222,41 @@ struct imgdata {			/* image loaded from a file */
    unsigned char *data;			/* pointer to image data */
    };
 
+struct imgmem {
+   int x, y, width, height;
+#ifdef XWindows
+   XImage *im;
+#endif					/* XWindows */
+#ifdef MSWindows
+   COLORREF *crp;
+#endif					/* MSWindows */
+   };
+
 #define TCH1 '~'			/* usual transparent character */
 #define TCH2 0377			/* alternate transparent character */
 #define PCH1 ' '			/* punctuation character */
 #define PCH2 ','			/* punctuation character */
 
 
+#ifdef MacGraph 
+typedef struct _wctype {
+   Pattern bkPat;
+   Pattern fillPat;
+   Point pnLoc;
+   Point pnSize;
+   short pnMode;
+   Pattern pnPat;
+   short txFont;
+   Style txFace;
+   short txMode;
+   short txSize;
+   Fixed spExtra;
+   RGBColor fgColor;
+   RGBColor bgColor;
+} ContextType, *ContextPtrType;
+#endif					/* MacGraph */
+
+
 #ifdef XWindows
 /*
  * Displays are maintained in a global list in rwinrsc.r.
@@ -302,9 +326,9 @@ typedef struct _wcontext {
   int		drawop;
   double	gamma;			/* gamma correction value */
   int		bits;			/* context bits */
-#ifdef MacGraph             /* MacGraph */
-  GrafPtr   contextPtr;     /* see InsideMac I-203 for this huge record */
-#endif                      /* MacGraph */
+#ifdef MacGraph
+  ContextPtrType   contextPtr;
+#endif					/* MacGraph */
 #ifdef XWindows
   wdp		display;
   GC		gc;			/* X graphics context */
@@ -365,11 +389,21 @@ typedef struct _wstate {
   word		timestamp;		/* last event time stamp */
   char		eventQueue[EQUEUELEN];  /* queue of cooked-mode keystrokes */
   int		eQfront, eQback;
+  char		*cursorname;
   struct descrip filep, listp;		/* icon values for this window */
-#ifdef MacGraph         /* MacGraph */
+#ifdef MacGraph
   WindowPtr theWindow;      /* pointer to the window */
   PicHandle windowPic;      /* handle to backing pixmap */
-#endif                  /* MacGraph */
+  GWorldPtr offScreenGWorld;  /* offscreen graphics world */
+  CGrafPtr   origPort;
+  GDHandle  origDev;
+  PixMapHandle offScreenPMHandle;
+  Rect      sourceRect;
+  Rect      destRect;
+  Rect      GWorldRect;
+  Boolean   lockOK;
+  Boolean   visible;
+#endif					/* MacGraph */
 #ifdef XWindows
   wdp		display;
   Window	win;			/* X window */
@@ -425,6 +459,8 @@ typedef struct _wstate {
   HBITMAP	iconpix;		/* backing bitmap */
   HBITMAP	initialPix;		/* backing bitmap */
   int		hasCaret;
+  HCURSOR	curcursor;
+  HCURSOR	savedcursor;
 #endif					/* MSWindows */
 } wstate, *wsp;
 
@@ -441,6 +477,18 @@ typedef struct _wbinding {
   wsp window;
 } wbinding, *wbp;
 
+#ifdef MacGraph
+typedef struct  
+   {
+   Boolean wasDown; 
+   uword when; 
+   Point where; 
+   int whichButton; 
+   int modKey; 
+   wsp ws;
+   } MouseInfoType;
+#endif					/* MacGraph */
+
 
 
 typedef struct {
