@@ -145,12 +145,18 @@ extern struct progstate rootpstate;
 extern int noMTevents;		/* no MT events during GC */
 #else					/* MultiThread */
 extern char *code;		/* start of icode */
+extern char *ecode;		/* end of icode */
 extern dptr statics; 		/* start of static variables */
 extern char *strcons;		/* start of the string constants */
 extern dptr fnames;		/* field names */
 extern dptr efnames;		/* end of field names */
 extern word *records;
 extern int *ftabp;		/* field table pointer */
+#ifdef FieldTableCompression
+extern word ftabwidth, foffwidth;
+extern unsigned char *ftabcp;
+extern short *ftabsp;
+#endif					/* FieldTableCompression */
 extern dptr xargp;
 extern word xnargs;
 
@@ -256,10 +262,12 @@ extern int keyboard_error;
 #endif					/* UNIX */
 
 #ifdef EventMon
+extern char typech[];
 extern word oldsum;
 extern struct descrip csetdesc;		/* cset descriptor */
 extern struct descrip eventdesc;	/* event descriptor */
 extern struct b_iproc mt_llist;
 extern struct descrip rzerodesc;	/* real descriptor */
 extern struct b_real realzero;		/* real zero block */
+
 #endif				/* EventMon */
