@@ -59,6 +59,10 @@
 #define LONG int
 #endif					/* LONG */
 
+/*
+ * Perform a "C" return, not processed by RTT
+ */
+#define VanquishReturn(s) return s;
 #endif					/* Graphics */
 
 /*
@@ -431,15 +435,9 @@
 /*
  * Macro for initialized procedure block.
  */
-#ifdef MultiThread
-#define B_IProc(n) struct {word title; word blksize; int (*ccode)();\
-   word nparam; word ndynam; word nstatic; word fstatic; struct progstate *p;\
-   struct sdescrip quals[n];}
-#else					/* MultiThread */
 #define B_IProc(n) struct {word title; word blksize; int (*ccode)();\
    word nparam; word ndynam; word nstatic; word fstatic;\
    struct sdescrip quals[n];}
-#endif					/* MultiThread */
 
 #define ssize    (curstring->size)
 #define strbase  (curstring->base)
@@ -556,18 +554,6 @@ static union {
 
 
 #if VMS
-#ifdef MultiThread
-#define FncBlock(f,nargs,deref) \
-	struct b_iproc Cat(B,f) = {\
-	T_Proc,\
-	Vsizeof(struct b_proc),\
-	Cat(Y,f),\
-	nargs,\
-	-1,\
-	deref, 0,\
-	0,\
-	{sizeof(Lit(f))-1,Lit(f)}};
-#else					/* MultiThread */
 #define FncBlock(f,nargs,deref) \
 	struct b_iproc Cat(B,f) = {\
 	T_Proc,\
@@ -577,20 +563,7 @@ static union {
 	-1,\
 	deref, 0,\
 	{sizeof(Lit(f))-1,Lit(f)}};
-#endif					/* MultiThread */
 #else					/* VMS */
-#ifdef MultiThread
-#define FncBlock(f,nargs,deref) \
-	struct b_iproc Cat(B,f) = {\
-	T_Proc,\
-	Vsizeof(struct b_proc),\
-	Cat(Z,f),\
-	nargs,\
-	-1,\
-	deref, 0,\
-	0,\
-	{sizeof(Lit(f))-1,Lit(f)}};
-#else					/* MultiThread */
 #define FncBlock(f,nargs,deref) \
 	struct b_iproc Cat(B,f) = {\
 	T_Proc,\
@@ -600,25 +573,11 @@ static union {
 	-1,\
 	deref, 0,\
 	{sizeof(Lit(f))-1,Lit(f)}};
-#endif					/* MultiThread */
 #endif					/* VMS */
 
 /*
  * Procedure block for an operator.
  */
-#ifdef MultiThread
-#define OpBlock(f,nargs,sname,xtrargs)\
-	struct b_iproc Cat(B,f) = {\
-	T_Proc,\
-	Vsizeof(struct b_proc),\
-	Cat(O,f),\
-	nargs,\
-	-1,\
-	xtrargs,\
-	0,\
-	0,\
-	{sizeof(sname)-1,sname}};
-#else					/* MultiThread */
 #define OpBlock(f,nargs,sname,xtrargs)\
 	struct b_iproc Cat(B,f) = {\
 	T_Proc,\
@@ -629,7 +588,6 @@ static union {
 	xtrargs,\
 	0,\
 	{sizeof(sname)-1,sname}};
-#endif					/* MultiThread */
 
 /*
  * Operator declaration.
@@ -698,8 +656,19 @@ static union {
 #define kywd_trc  (curpstate->Kywd_trc)
 #define mainhead (curpstate->Mainhead)
 #define code (curpstate->Code)
+#define ecode (curpstate->Ecode)
 #define records (curpstate->Records)
 #define ftabp (curpstate->Ftabp)
+#ifdef FieldTableCompression
+#define ftabwidth (curpstate->Ftabwidth)
+#define foffwidth (curpstate->Foffwidth)
+#define ftabcp (curpstate->Ftabcp)
+#define ftabsp (curpstate->Ftabsp)
+#define focp (curpstate->Focp)
+#define fosp (curpstate->Fosp)
+#define fo (curpstate->Fo)
+#define bm (curpstate->Bm)
+#endif					/* FieldTableCompression */
 #define fnames (curpstate->Fnames)
 #define efnames (curpstate->Efnames)
 #define globals (curpstate->Globals)
@@ -756,6 +725,7 @@ static union {
 #define coll_blk  (curpstate->collblk)
 
 #define lastop    (curpstate->Lastop)
+#define lastopnd    (curpstate->Lastopnd)
 
 #define xargp     (curpstate->Xargp)
 #define xnargs    (curpstate->Xnargs)
