@@ -57,9 +57,12 @@ int	doasgn			Params((dptr dp1,dptr dp2));
 int	doimage			Params((int c,int q));
 int	equiv			Params((dptr dp1,dptr dp2));
 int	err			Params((noargs));
+int	findcol			Params((word *ipc));
 int	findipc			Params((int line));
 int	findline		Params((word *ipc));
+int	findloc			Params((word *ipc));
 int	getstrg			Params((char *buf,int maxi,FILE *fd));
+int	idelay			Params((int n)); 
 int	lexcmp			Params((dptr dp1,dptr dp2));
 int	numcmp			Params((dptr dp1,dptr dp2,dptr dp3));
 int	pushact		Params((struct b_coexpr *ce, struct b_coexpr *actvtr));
@@ -76,6 +79,7 @@ novalue	corettrace	Params((struct b_coexpr *ccp,struct b_coexpr *ncp));
 novalue	datainit		Params((noargs));
 novalue	deallocate		Params((union block *bp));
 novalue	drunerr			Params((int n, double v));
+novalue	dumpact			Params((struct b_coexpr *ce));
 novalue	env_int	Params((char *name,word *variable,int non_neg, uword limit));
 novalue	error			Params((char *s1, char *s2));
 novalue	err_msg			Params((int n, dptr v));
@@ -173,10 +177,8 @@ char	*sbrk	();
 #endif					/* ExecImages */
 #endif                                  /* UNIX || VMS */
 
-#ifdef Coexpr
+/* the following is just a stub in implementations without Coexpr */
 novalue new_context		Params((int fsig, dptr cargp));
-#endif					/* Coexpr */
-
 
 #ifdef Graphics
 /*
@@ -186,13 +188,13 @@ wcp	alc_context		Params((wbp w));
 wbp	alc_wbinding		Params((noargs));
 wsp	alc_winstate		Params((noargs));
 int	atobool			Params((char *s));
+int	c_push			Params((dptr l,dptr val));  /* in fstruct.r */
 int	docircles		Params((wbp w, int argc, dptr argv, int fill));
 novalue	drawCurve		Params((wbp w, XPoint *p, int n));
 char	*evquesub		Params((wbp w, int i));
 wsp	getactivewindow		Params((void));
 novalue	genCurve		Params((wbp w, XPoint *p, int n, void (*h)()));
 int	getpattern		Params((wbp w, char *answer));
-novalue	mystrncpy		Params((char *dest,char *src,int n));
 struct	palentry *palsetup	Params((int p));
 int	palnum			Params((dptr d));
 int	parsecolor		Params((wbp w, char *s, long *r, long *g, long *b));
@@ -209,7 +211,6 @@ int	writeGIF		Params((wbp w, char *filename,
 int	setsize			Params((wbp w, char *s));
 char	*si_i2s			Params((siptr sip, int i));
 int	si_s2i			Params((siptr sip, char *s));
-char	*strnchr		Params((char *s, int ci, long len));
 int	ulcmp			Params((pointer p1, pointer p2));
 int	wattrib			Params((wbp w, char *s, long len, dptr answer, char *abuf));
 int	wgetche			Params((wbp w, dptr res));
@@ -254,8 +255,9 @@ int	geticonpos		Params((wbp w, char *s));
 int	getimstr		Params((wbp w, int x, int y, int width, int hgt,
 				  struct palentry *ptbl, unsigned char *data));
 novalue	getlinestyle		Params((wbp w, char *answer));
-int	getpixel_init		Params((wbp w, int x, int y, int width, int height));
-int	getpixel		Params((wbp w, int x, int y, long *rv, char *s));
+int	getpixel_init		Params((wbp w, struct imgmem *imem));
+int	getpixel_term		Params((wbp w, struct imgmem *imem));
+int	getpixel		Params((wbp w, int x, int y, long *rv, char *s, struct imgmem *imem));
 novalue	getpointername		Params((wbp w, char *answer));
 int	getpos			Params((wbp w));
 int	getvisual		Params((wbp w, char *answer));
@@ -312,6 +314,7 @@ FILE	*wopen			Params((char *windowname, struct b_list *hp, dptr attr, int n, int
 int	wputc			Params((int ci, wbp w));
 novalue	wsync			Params((wbp w));
 novalue	xdis			Params((wbp w, char *s, int n));
+
 #ifdef ConsoleWindow
 FILE* OpenConsole		Params((void));
 int   Consolefprintf		(FILE *file, char *format, ...);
@@ -333,6 +336,7 @@ void DoMouseDown (EventRecord *eventPtr);
 void DoGrowWindow (EventRecord *eventPtr, WindowPtr whichWindow);
 void GetLocUpdateRgn (WindowPtr whichWindow, RgnHandle localRgn);
 void DoKey (EventRecord *eventPtr, WindowPtr whichWindow);
+void EventLoop(void);
 void HandleMenuChoice (long menuChoice);
 void HandleAppleChoice (short item);
 void HandleFileChoice (short item);
@@ -340,11 +344,18 @@ void HandleOptionsChoice (short item);
 void DoUpdate (EventRecord *eventPtr);
 void DoActivate (WindowPtr whichWindow, Boolean becomingActive);
 void RedrawWindow (WindowPtr whichWindow);
-/*void setfile(char *filename, OSType type, OSType creator);*/
 const int ParseCmdLineStr (char *s, char *t, char **argv);
 pascal OSErr SetDialogDefaultItem (DialogPtr theDialog, short newItem) = { 0x303C, 0x0304, 0xAA68 };
 pascal OSErr SetDialogCancelItem (DialogPtr theDialog, short newItem) = { 0x303C, 0x0305, 0xAA68 };
 pascal OSErr SetDialogTracksCursor (DialogPtr theDialog, Boolean tracks) = { 0x303C, 0x0306, 0xAA68 };
+
+void drawarcs(wbinding *wb, XArc *arcs, int narcs);
+void drawlines(wbinding *wb, XPoint *points, int npoints);  
+void drawpoints(wbinding *wb, XPoint *points, int npoints);  
+void drawrectangles(wbp wb, XRectangle *recs, int nrecs);
+void drawsegments(wbinding *wb, XSegment *segs, int nsegs);
+void fillarcs(wbp wb, XArc *arcs, int narcs);
+void fillpolygon(wbp wb, XPoint *pts, int npts);  
 #endif					/* MacGraph */
 
 #ifdef XWindows
@@ -556,10 +567,6 @@ struct	b_proc *strprc	Params((dptr s, C_integer arity));
 struct	b_tvsubs *alcsubs	Params((word len,word pos,dptr var));
 
 #ifdef EventMon
-novalue	MMAlc	Params((word len, int type));
-novalue	MMStr	Params((word slen));
-novalue EVVal	Params((word value, int event));
-novalue EVValD	Params((dptr dp, int event));
 novalue EVAsgn	Params((dptr dx));
 #endif					/* EventMon */
 
