@@ -99,10 +99,6 @@ struct b_proc {			/* procedure block */
    word nstatic;		/*   number of static locals */
    word fstatic;		/*   index (in global table) of first static */
 
-#ifdef MultiThread
-   struct progstate *program;
-#endif				/* MultiThread */
-
    struct descrip pname;	/*   procedure name (string qualifier) */
    struct descrip lnames[1];	/*   list of local names (qualifiers) */
    };
@@ -363,8 +359,16 @@ struct progstate {
    struct descrip Kywd_trc;
    struct b_coexpr *Mainhead;
    char *Code;
+   char *Ecode;
    word *Records;
    int *Ftabp;
+#ifdef FieldTableCompression
+   short Ftabwidth, Foffwidth;
+   unsigned char *Ftabcp, *Focp;
+   short *Ftabsp, *Fosp;
+   int *Fo;
+   char *Bm;
+#endif					/* FieldTableCompression */
    dptr Fnames, Efnames;
    dptr Globals, Eglobals;
    dptr Gnames, Egnames;
@@ -491,10 +495,6 @@ struct b_iproc {		/* procedure block */
    word ip_nstatic;		/*   number of static locals */
    word ip_fstatic;		/*   index (in global table) of first static */
 
-#ifdef MultiThread
-   struct progstate *ip_program;
-#endif				/* MultiThread */
-
    struct sdescrip ip_pname;	/*   procedure name (string qualifier) */
    struct descrip ip_lnames[1];	/*   list of local names (qualifiers) */
    };
