@@ -19,13 +19,13 @@
 /*
  * &version
  */
-#define Version	"Icon Compiler Version 9.1.  November 21, 1995"
+#define Version	"Icon Version 9.3c.  November 25, 1996"
 #else					/* COMPILER */
 
 /*
  *  &version
  */
-#define Version	"Icon Interpreter Version 9.1.  November 21, 1995"
+#define Version	"Icon Version 9.3.  November 25, 1996"
 
 /*
  * Version numbers to be sure ucode is compatible with the linker
@@ -34,21 +34,21 @@
 
 #define UVersion "U9.0.00"
 
-#ifdef MultiThread
+#ifdef FieldTableCompression
 
 #if IntBits == 16
-#define IVersion "I9.0.00M/16"
+#define IVersion "I9.2.00FT/16"
 #endif					/* IntBits == 16 */
 
 #if IntBits == 32
-#define IVersion "I9.0.00M/32"
+#define IVersion "I9.2.00FT/32"
 #endif					/* IntBits == 32 */
 
 #if IntBits == 64
-#define IVersion "I9.0.00M/64"
+#define IVersion "I9.2.00FT/64"
 #endif					/* IntBits == 64 */
 
-#else					/* MultiThread */
+#else					/* FieldTableCompression */
 
 #if IntBits == 16
 #define IVersion "I9.0.00/16"
@@ -62,8 +62,7 @@
 #define IVersion "I9.0.00/64"
 #endif					/* IntBits == 64 */
 
-#endif					/* MultiThread */
-
+#endif					/* FieldTableCompression */
 #endif					/* COMPILER */
 
 /*
