@@ -10,6 +10,10 @@
 #include "ctree.h"
 #include "cproto.h"
 
+#ifndef LoopThreshold
+#define LoopThreshold 7
+#endif					/* LoopThreshold */
+
 /*
  * MinOne - arrays sizes must be at least 1.
  */
@@ -378,7 +382,7 @@ int prt;
    }
 
 /*
- * dyn_nams - compute offsets of dynamic locals and, if needed, output the
+ * dyn_nms - compute offsets of dynamic locals and, if needed, output the
  *  initializer for a descriptor for the variable name.
  */
 static int dyn_nms(lptr, prt)
@@ -651,6 +655,9 @@ struct c_fnc *fnc;
    int nparms;
    int optim; /* optimized interface: no arg list adjustment */
    int ret_flag;
+#ifdef OptimizeLoop
+   int i;
+#endif					/* OptimizeLoop */
 
    prefix = cur_proc->prefix;
    name = cur_proc->name;
@@ -729,8 +736,20 @@ struct c_fnc *fnc;
    else
       fprintf(codefile, "   r_frame.succ_cont = NULL;\n");
    fprintf(codefile, "\n");
+#ifdef OptimizeLoop
+   if (ntend > 0)  {
+      if (ntend < LoopThreshold)
+         for (i=0; i < ntend ;i++)  
+            fprintf(codefile, "   r_frame.tend.d[%d] = nulldesc;\n", i);
+      else {
+         fprintf(codefile, "   for (i = 0; i < %d; ++i)\n", ntend);
+         fprintf(codefile, "      r_frame.tend.d[i] = nulldesc;\n");
+      }
+   }
+#else					/* OptimizeLoop */
    fprintf(codefile, "   for (i = 0; i < %d; ++i)\n", ntend);
    fprintf(codefile, "      r_frame.tend.d[i] = nulldesc;\n");
+#endif					/* OptimizeLoop */
    if (optim) {
       /*
        * Dereferencing and argument list adjustment is done at the call
@@ -772,6 +791,13 @@ struct c_fnc *fnc;
           * Output code to dereference argument or supply default null
           *  value.
           */
+#ifdef OptimizeLoop
+         fprintf(codefile, "   for (i = 0; i < r_nargs ; ++i)\n");
+         fprintf(codefile, "      deref(&r_args[i], &r_frame.tend.d[i + %d]);\n",  first_arg);
+         fprintf(codefile, "   for(i = r_nargs; i < %d ; ++i)\n", nparms);
+         fprintf(codefile, "      r_frame.tend.d[i + %d] = nulldesc;\n",
+            first_arg);
+#else					/* OptimizeLoop */
          fprintf(codefile, "   for (i = 0; i < %d; ++i)\n", nparms);
          fprintf(codefile, "      if (i < r_nargs)\n");
          fprintf(codefile,
@@ -780,6 +806,7 @@ struct c_fnc *fnc;
          fprintf(codefile, "      else\n");
          fprintf(codefile, "         r_frame.tend.d[i + %d] = nulldesc;\n",
             first_arg);
+#endif					/* OptimizeLoop */
          }
       fprintf(codefile, "   argp = &r_frame.tend.d[%d];\n", first_arg);
       }
