@@ -997,7 +997,7 @@ int op;
  *  bogus results (as well as reams of filen commands).
  */
 static char *lastfiln = NULL;
-static int lastline = 0;
+static int lastlin = 0;
 
 static novalue setloc(n)
 nodeptr n;
@@ -1014,8 +1014,8 @@ nodeptr n;
    /*
     * if either line or column has changed, emit location information
     */
-   if (((Col(n) << 16) + Line(n)) != lastline) {
-      lastline = (Col(n) << 16) + Line(n);
+   if (((Col(n) << 16) + Line(n)) != lastlin) {
+      lastlin = (Col(n) << 16) + Line(n);
       emitn("line",Line(n));
       emitn("colm",Col(n));
       }
@@ -1023,9 +1023,9 @@ nodeptr n;
    /*
     * if line has changed, emit line information
     */
-   if (Line(n) != lastline) {
-      lastline = Line(n);
-      emitn("line", lastline);
+   if (Line(n) != lastlin) {
+      lastlin = Line(n);
+      emitn("line", lastlin);
       }
 #endif					/* SrcColumnInfo */
 
