@@ -3,6 +3,9 @@
  */
 
 #include "../h/gsupport.h"
+#undef T_Real
+#undef T_String
+#undef T_Cset
 #include "../h/lexdef.h"
 #include "ttoken.h"
 #include "tree.h"
