@@ -5,38 +5,56 @@
 #include "../h/gsupport.h"
 #include "tproto.h"
 
+#if SCCX_MX
+#include "../h/filepat.h"
+
+/* This sets the stack size so it runs in a Windows DOS box */
+unsigned _stack = 100000;
+#endif      /* SCCX_MX */
+
 #if MACINTOSH
 #if THINK_C
+#include <string.h>
 #include "console.h"
+#include "config.h"
+#include "cpuconf.h"
+#include "macgraph.h"
 #include <AppleEvents.h>
 #include <GestaltEqu.h>
-#include <Values.h>
+/* #include <Values.h> */
 
-#define kSleep MAXLONG
+#define MAXLONG  (0x7fffffff)
+
+/* #define kSleep MAXLONG */
 #define kGestaltMask 1L
 
 /* Global */
 Boolean gDone;
+Boolean g_cOption;
+
 
 #endif    /* THINK_C */
 #endif    /* MACINTOSH */
 
-#if NT || MICROSOFT
-#include <io.h>
-#endif					/* NT || MICROSOFT */
+#ifdef MSWindows
+#ifdef NTConsole
+#define int_PASCAL int PASCAL
+#define LRESULT_CALLBACK LRESULT CALLBACK
+#include <windows.h>
+#include "../h/filepat.h"
+#endif					/* NTConsole */
+#endif					/* MSWindows */
 
 /*
  * Prototypes.
  */
 
 hidden	novalue	execute	Params((char *ofile,char *efile,char * *args));
-hidden	novalue	report Params((char *s));
 hidden	novalue	rmfiles Params((char **p));
 hidden	novalue	usage Params((noargs));
 
 #if MACINTOSH
 #if THINK_C
-pascal void  SetApplLimit  ( void *zoneLimit );
 pascal void  MaxApplZone ( void );
 void         IncreaseStackSize ( Size extraBytes );
 
@@ -51,9 +69,16 @@ pascal OSErr DoQuitApp ( AppleEvent theAppleEvent,
                          AppleEvent reply,
                                long refCon );
 OSErr        GotRequiredParams ( AppleEvent *appleEventPtr );
-novalue      MacMain ( int argc, char **argv );             
-#endif                  /* THINK_C */
-#endif                  /* MACINTOSH */
+novalue      MacMain ( int argc, char **argv );
+
+void DoMouseDown (EventRecord *eventPtr);
+void HandleMenuChoice (long menuChoice);
+void HandleAppleChoice (short item);
+void HandleFileChoice (short item);
+void HandleOptionsChoice (short item);
+void MenuBarInit (void);
+#endif					/* THINK_C */
+#endif					/* MACINTOSH */
 
 /*
  * The following code is operating-system dependent [@tmain.01].  Include
@@ -147,7 +172,7 @@ int ConsolePause = 1;
 #endif					/* ConsoleWindow */
 
 
- 
+
 #if MACINTOSH
 #if THINK_C
 void main ( void )
@@ -158,12 +183,13 @@ void main ( void )
    Size extraBytes=32*1024;
    IncreaseStackSize (extraBytes);
  */
- 
+
    MaxApplZone();
 
    ToolBoxInit ();
    EventInit ();
-   
+   MenuBarInit();
+
    EventLoop ();
 }                    /* end of main () */
 #endif               /* THINK_C */
@@ -198,7 +224,7 @@ void expand_proj(int *argc, char ***argv)
    if(fgets(buf, 1023, f) != NULL) {
       if (strchr(buf, '\n') != NULL)
          buf[strlen(buf)-1] = '\0';
-      (*argv)[k++] = strdup(buf);
+      (*argv)[k++] = salloc(buf);
       while(fgets(buf, 1023, f) != NULL) {
          if (strchr(buf, '\n') != NULL)
             buf[strlen(buf)-1] = '\0';
@@ -208,7 +234,7 @@ void expand_proj(int *argc, char ***argv)
             fflush(stderr);
             return;
             }
-         (*argv)[k++] = strdup(buf);
+         (*argv)[k++] = salloc(buf);
          }
       }
    fclose(f);
@@ -252,19 +278,24 @@ int CmdParamToArgv(char *s, char ***avp)
       if (*t2) *t2++ = '\0';
       }
    (*avp)[rv] = NULL;
-   return rv;  
+   return rv;
    }
 
 
 LRESULT_CALLBACK WndProc	Params((HWND, UINT, WPARAM, LPARAM));
+char *lognam;
+char tmplognam[128];
+extern FILE *flog;
 
 novalue MSStartup(int argc, char **argv, HINSTANCE hInstance, HINSTANCE hPrevInstance)
    {
    WNDCLASS wc;
-   extern FILE *flog;
    /* generate log file?  might make this an option */
-   if (0)
-      flog = fopen("wicont.log", "w");
+   if ((lognam = getenv("ICONTLOG"))!=NULL) {
+      remove(lognam);
+      lognam = strdup(lognam);
+      flog = fopen(tmpnam(tmplognam), "w");
+      }
    if (!hPrevInstance) {
 #if NT
       wc.style = CS_HREDRAW | CS_VREDRAW;
@@ -341,12 +372,10 @@ char **argv;
    char ch;
    struct fileparts *fp;
 
-#if NT || MICROSOFT
-   struct _finddata_t fd;
+#if WildCards
+   FINDDATA_T fd;
    int j;
-   long l;
-#endif					/* NT || MICROSOFT */
-
+#endif					/* WildCards */
 
 #if SCCX_MX
     syntax error!
@@ -437,6 +466,11 @@ char **argv;
          case 'I':			/* -C */
             makeExe = 0;
             break;
+#ifdef SCCX_MX
+         case 'A':
+            makeExe = 2;
+            break;
+#endif                  /* SCCX_MX */
 #endif					/* MSDOS */
 
          case 'c':			/* -c: compile only (no linking) */
@@ -516,7 +550,7 @@ char **argv;
                "Can't understand what directory icont was run from.\n");
             exit(ErrorExit);
             }
-            strcpy (++pathCursor, "ixhdr.exe");
+            strcpy( ++pathCursor, (makeExe==1) ?  "ixhdr.exe" : "iconx.exe");
          }
 #endif                                  /* MSDOS && !NT */
 
@@ -525,20 +559,15 @@ char **argv;
     */
    n = argc - optindex + 1;
 
-#if NT || MICROSOFT
-   /*
-    * allocate additional space for wildcarded names, if any
-    */
+#if WildCards
    {
    for(j=optindex; j < argc; j++) {
-      l = _findfirst(argv[j], &fd);
-      if (l != -1) {
-         while(!_findnext(l, &fd)) n++;
-	  _findclose(l);
-	 }
+      if (FINDFIRST(argv[j], &fd))
+         while(FINDNEXT(&fd)) n++;
+      FINDCLOSE(&fd);
       }
    }
-#endif					/* NT || MICROSOFT */
+#endif					/* WildCards */
 
    tptr = tfiles = (char **)alloc((unsigned int)(n * sizeof(char *)));
    lptr = lfiles = (char **)alloc((unsigned int)(n * sizeof(char *)));
@@ -570,31 +599,31 @@ char **argv;
 
          }
       else {
-#if NT || MICROSOFT
-         char tmp[MaxPath];
-         char *p;
-	 strcpy(tmp, argv[optindex]);
-	 if ((p = strrchr(tmp, '.')) == NULL ||
-	      strchr(p, '/') || strchr(p, '\\')) {
-	    strcat(tmp,".icn");
+#if WildCards
+	 char tmp[MaxPath], dir[MaxPath];
+         int matches = 0;
+
+	 fp = fparse(argv[optindex]);
+	 /* Save because *fp will get overwritten frequently */
+	 strcpy(dir, fp->dir);
+	 if (*fp->ext == '\0')
+	    makename(tmp,NULL,argv[optindex], SourceSuffix);
+	 else
+	    strcpy(tmp, argv[optindex]);
+
+	 if (strchr(tmp, '*') || strchr(tmp, '?')) {
+	    if (!FINDFIRST(tmp, &fd)) {
+	       fprintf(stderr, "File %s: no match\n", tmp);
+	       fflush(stderr);
+	       }
+	    else matches = 1;
 	    }
-         l = _findfirst(tmp, &fd);
-	 if (l == -1) {
-	    fprintf(stderr,"File %s: no match\n", argv[optindex]);
-	    fflush(stderr);
-	    exit(ErrorExit);
-	    }
-	 if ((p = strrchr(tmp, '\\')) || (p = strrchr(tmp, '//')) ||
-	     (p = strrchr(tmp, ':')))
-	     p++;
          do {
-         if (p) {
-            strcpy(p, fd.name);
-            argv[optindex] = tmp;
-            }
-         else
-            argv[optindex] = fd.name;
-#endif					/* NT || MICROSOFT */
+	 if (matches) {
+	    makename(tmp,dir,FILENAME(&fd),NULL);
+	    argv[optindex] = tmp;
+	    }
+#endif					/* WildCards */
          fp = fparse(argv[optindex]);		/* parse file name */
          if (*fp->ext == '\0' || smatch(fp->ext, SourceSuffix)) {
             makename(buf,SourceDir,argv[optindex], SourceSuffix);
@@ -614,14 +643,16 @@ char **argv;
             }
          else
             quitf("bad argument %s",argv[optindex]);
-#if NT || MICROSOFT
-         } while (!_findnext(l, &fd));
-	 _findclose(l);
-#endif					/* NT || MICROSOFT */
+#if WildCards
+	 if (!matches)
+	    break;
+         } while (FINDNEXT(&fd));
+	 FINDCLOSE(&fd);
+#endif					/* WildCards */
          }
       optindex++;
       }
- 
+
    *tptr = *lptr = *rptr = NULL;	/* terminate filename lists */
    if (lptr == lfiles)
       usage();				/* error -- no files named */
@@ -639,7 +670,7 @@ char **argv;
     * Translate .icn files to make .u1 and .u2 files.
     */
    if (tptr > tfiles) {
-      if (!silent && !pponly)
+      if (!pponly)
          report("Translating");
       errors = trans(tfiles);
       if (errors > 0)			/* exit if errors seen */
@@ -671,14 +702,17 @@ char **argv;
 
 #if MSDOS
 #if NT
+   {
    if (ofile == NULL)  {                /* if no -o file, synthesize a name */
       ofile = salloc(makename(buf,TargetDir,lfiles[0],
-                              makeExe ? ".cmd" : IcodeSuffix));
-   } else {                             /* add extension if necessary */
+                              makeExe ? ".bat" : IcodeSuffix));
+      }
+   else {                             /* add extension if necessary */
       fp = fparse(ofile);
       if (*fp->ext == '\0' && *IcodeSuffix != '\0') /* if no ext given */
          ofile = salloc(makename(buf,NULL,ofile,
-                                 makeExe ? ".cmd" : IcodeSuffix));
+                                 makeExe ? ".bat" : IcodeSuffix));
+      }
    }
 #else					/* NT */
    if (ofile == NULL)  {                /* if no -o file, synthesize a name */
@@ -704,8 +738,7 @@ char **argv;
 
 #endif					/* MSDOS */
 
-   if (!silent)
-      report("Linking");
+   report("Linking");
    errors = ilink(lfiles,ofile);	/* link .u files to make icode file */
 
    /*
@@ -727,14 +760,12 @@ char **argv;
       }
 #ifdef ConsoleWindow
    else
-      if (!silent)
-         fprintf(stderr, "No errors\n");
+      report("No errors\n");
 #endif					/* ConsoleWindow */
 
 #if !(MACINTOSH && MPW)
    if (optindex < argc)  {
-      if (!silent)
-         report("Executing");
+      report("Executing");
       execute (ofile, efile, argv+optindex+1);
       }
 #endif					/* !(MACINTOSH && MPW) */
@@ -772,6 +803,7 @@ char *ofile, *efile, **args;
    }
 #else					/* AMIGA && LATTICE */
 #ifdef MSWindows
+#ifndef NTConsole
    {
      char cmdline[256], *tmp;
 
@@ -791,6 +823,7 @@ char *ofile, *efile, **args;
    WinExec(cmdline, SW_SHOW);
    return;
    }
+#endif					/* NTConsole */
 #endif					/* MSWindows */
 
 #if SCCX_MX
@@ -823,7 +856,7 @@ Deliberate Syntax Error
 #if LATTICE
       {
       struct ProcID procid;
-      if (forkv(iconxloc,argv,NULL,&procid) == 0) { 
+      if (forkv(iconxloc,argv,NULL,&procid) == 0) {
          wait(&procid);
          return;
          }
@@ -899,7 +932,7 @@ Deliberate Syntax Error
 #if UNIX
       /*
        * If an ICONX environment variable is defined, use that.
-       *  If not, first try the predefined path, then search $PATH via execvp. 
+       *  If not, first try the predefined path, then search $PATH via execvp.
        */
       if ((argv[0] = getenv("ICONX")) != NULL && argv[0][0] != '\0') {
          execv(argv[0], argv);	/* exec file specified by $ICONX */
@@ -942,12 +975,16 @@ Deliberate Syntax Error
 
    }
 
-static novalue report(s)
+novalue report(s)
 char *s;
    {
-
-   fprintf(stderr,"%s:\n",s);
-
+   char *c = (strchr(s, '\n') ? "" : ":\n") ;
+   if (!silent)
+      fprintf(stderr,"%s%s",s,c);
+#ifdef ConsoleWindow
+   else if (flog != NULL)
+      fprintf(flog, "%s%s",s,c);
+#endif					/* ConsoleWindow */
    }
 
 /*
@@ -997,7 +1034,7 @@ void IncreaseStackSize (Size extraBytes)
  */
 void ToolBoxInit ( void )
 {
-   InitGraf ( &thePort );
+   InitGraf ( &qd.thePort );
    InitFonts ();
    InitWindows ();
    InitMenus ();
@@ -1006,6 +1043,183 @@ void ToolBoxInit ( void )
    InitCursor ();
 }
 
+/*-------------- code for display graphics ----------------------*/
+
+/*
+ * DoEvent - handles AppleEvent by passing eventPtr to AEProcessAppleEvent
+ *           Command-Q key sequence results exit of the program
+ */
+void DoEvent ( EventRecord *eventPtr )
+{
+   char theChar;
+
+   switch ( eventPtr->what )
+   {
+      case mouseDown:        DoMouseDown (eventPtr);
+                             break;
+/*
+      case kHighLevelEvent : AEProcessAppleEvent ( eventPtr );
+                             break;
+ */
+      case keyDown:
+      case autoKey:          theChar = eventPtr->message & charCodeMask;
+                             if ( (eventPtr->modifiers & cmdKey) != 0)
+                                HandleMenuChoice (MenuKey (theChar));
+                             break;
+   }
+}
+
+/*
+ * DoMouseDown -
+ */
+void DoMouseDown (EventRecord *eventPtr)
+{
+   WindowPtr whichWindow;
+   short thePart;
+   long menuChoice;
+
+   thePart = FindWindow (eventPtr->where, &whichWindow);
+   switch (thePart) {
+      case inMenuBar:
+         menuChoice = MenuSelect (eventPtr->where);
+         HandleMenuChoice (menuChoice);
+         break;
+      }
+}
+
+/*
+ * HandleMenuChoice -
+ */
+void HandleMenuChoice (long menuChoice)
+{
+   short menu;
+   short item;
+
+   if (menuChoice != 0) {
+      menu = HiWord (menuChoice);
+      item = LoWord (menuChoice);
+
+      switch (menu) {
+         case kAppleMenu:
+            HandleAppleChoice (item);
+            break;
+         case kFileMenu:
+            HandleFileChoice (item);
+            break;
+         case kOptionsMenu:
+            HandleOptionsChoice (item);
+            break;
+         }
+      HiliteMenu (0);
+      }
+}
+
+void HandleAppleChoice (short item)
+{
+   MenuHandle  appleMenu;
+   Str255      accName;
+   short       accNumber;
+
+   switch (item) {
+      case kAboutMItem:
+         SysBeep (20);
+         break;
+         /* ******************* open a dialog box **************** */
+      default:
+         appleMenu = GetMHandle (kAppleMenu);
+         GetItem (appleMenu, item, accName);
+         accNumber = OpenDeskAcc (accName);
+         break;
+      }
+}
+
+void HandleFileChoice (short item)
+{
+   StandardFileReply fileReply;
+   SFTypeList typeList;
+   short numTypes;
+   char *fileName;
+   int argc;
+   char **argv;
+   MenuHandle menu;
+
+   switch (item) {
+      case kQuitMItem:
+         gDone = true;
+         abort ();
+         break;
+      case kCompileMItem:
+         typeList[0] = 'TEXT';
+         numTypes = 1;
+         StandardGetFile (nil, numTypes, typeList, &fileReply);
+         if (fileReply.sfGood) {
+            fileName = PtoCstr (fileReply.sfFile.name);
+            menu = GetMHandle (kFileMenu);
+            DisableItem (menu, kCompileMItem);
+            menu = GetMHandle (kOptionsMenu);
+            DisableItem (menu, 0);
+            }
+         else
+            break;
+
+         if (g_cOption)
+            argc = 3;
+         else
+            argc = 2;
+
+         argv = malloc (sizeof (*argv) * argc);
+         argv[0] = malloc (strlen ("ICONT") + 1);
+         strcpy (argv[0], "ICONT");
+
+         if (g_cOption) {
+            argv[1] = malloc (strlen ("-c") + 1);
+            strcpy (argv[1], "-c");
+            }
+
+         argv[argc-1] = malloc (strlen(fileName) + 1);
+         strcpy (argv[argc-1], fileName);
+
+         MacMain (argc, argv);
+         break;
+      }
+}
+
+void HandleOptionsChoice (short item)
+{
+   MenuHandle menu;
+
+   switch (item) {
+      case k_cMItem:
+         g_cOption = !g_cOption;
+         menu = GetMHandle (kOptionsMenu);
+         CheckItem (menu, item, g_cOption);
+         break;
+      }
+}
+
+/*------------------  End of display graphics code ------------------------*/
+
+void MenuBarInit (void)
+{
+   Handle         menuBar;
+   MenuHandle     menu;
+   OSErr          myErr;
+   long           feature;
+
+   g_cOption = false;
+
+   menuBar = GetNewMBar (kMenuBar);
+   SetMenuBar (menuBar);
+
+   menu = GetMHandle (kAppleMenu);
+   AddResMenu (menu, 'DRVR');
+
+   menu = GetMHandle (kOptionsMenu);
+   CheckItem (menu, k_cMItem, g_cOption);
+
+   DrawMenuBar ();
+}
+
 /*
  * EventInit - calls Gestalt to check if AppleEvent is available, if so,
  *   install OpenDocument and QuitApplication handler routines
@@ -1014,9 +1228,9 @@ void EventInit ( void )
 {
    OSErr err;
    long  feature;
-   
+
    err = Gestalt ( gestaltAppleEventsAttr, &feature );
-   
+
    if ( err != noErr ) {
       printf ("Problem in calling Gestalt.");
       return;
@@ -1027,19 +1241,19 @@ void EventInit ( void )
          return;
          }
       }
-   
+
    err = AEInstallEventHandler (kCoreEventClass,
                                 kAEOpenDocuments,
-                                DoOpenDoc,
+            (AEEventHandlerUPP)   DoOpenDoc,
                                 0L,
                                 false );
-                                
+
    if ( err != noErr )
       printf ("kAEOpenDocuments Apple Event not available!");
-   
+
    err = AEInstallEventHandler (kCoreEventClass,
                                 kAEQuitApplication,
-                                DoQuitApp,
+             (AEEventHandlerUPP)  DoQuitApp,
                                 0L,
                                 false );
    if ( err != noErr )
@@ -1052,7 +1266,7 @@ void EventInit ( void )
 void EventLoop ( void )
 {
    EventRecord event;
-   
+
    gDone = false;
    while ( gDone == false ) {
       if ( WaitNextEvent ( everyEvent, &event, kSleep, nil ) )
@@ -1060,28 +1274,6 @@ void EventLoop ( void )
       }
 }
 
-/*
- * DoEvent - handles AppleEvent by passing eventPtr to AEProcessAppleEvent
- *           Command-Q key sequence results exit of the program
- */
-void DoEvent ( EventRecord *eventPtr )
-{
-   char theChar;
-   
-   switch ( eventPtr->what ) {
-      case kHighLevelEvent :
-	 AEProcessAppleEvent ( eventPtr );
-	 break;
-      case keyDown:
-      case autoKey:
-	 theChar = eventPtr->message & charCodeMask;
-	 if ( ( eventPtr->modifiers & cmdKey ) != 0 )
-	    if ( ( theChar=='Q' )  | ( theChar=='q' ) )
-	       gDone = true;
-	 break;
-      }
-}
-
 /*
  * DoOpenDoc - called by AEProcessAppleEvent (a ToolBox routine)
  *
@@ -1101,12 +1293,12 @@ pascal OSErr DoOpenDoc ( AppleEvent theAppleEvent,
    long       count;
    AEKeyword  keyword;
    long       i;
-   
+
    int        argc;
    char       **argv;
-   
+
    char       *fileName;
-   
+
    err = AEGetParamDesc ( &theAppleEvent,
                           keyDirectObject,
                           typeAEList,
@@ -1115,13 +1307,13 @@ pascal OSErr DoOpenDoc ( AppleEvent theAppleEvent,
       printf ("Error AEGetParamDesc");
       return ( err );
       }
-   
+
    err = GotRequiredParams ( &theAppleEvent );
    if ( err != noErr ) {
       printf ("Error GotRequiredParams");
       return ( err );
       }
-   
+
    err = AECountItems ( &fileSpecList, &count );
    if ( err != noErr ) {
       printf ("Error AECountItems");
@@ -1132,7 +1324,7 @@ pascal OSErr DoOpenDoc ( AppleEvent theAppleEvent,
    argv = malloc (sizeof (*argv) * (argc + 1));
    argv[0] = malloc (strlen("ICONT") + 1);
    strcpy (argv[0], "ICONT");
-   
+
    for ( i = 1; i <= count; i++ ) {
       err = AEGetNthPtr ( &fileSpecList,
                           i,
@@ -1146,7 +1338,7 @@ pascal OSErr DoOpenDoc ( AppleEvent theAppleEvent,
          printf ("Error AEGetNthPtr");
          return ( err );
 	 }
-      
+
       fileName = PtoCstr (file.name);
       argv[i] = malloc(strlen(fileName) + 1);
       strcpy (argv[i], fileName);
@@ -1167,7 +1359,7 @@ pascal OSErr DoQuitApp ( AppleEvent theAppleEvent,
    OSErr err = noErr;
    gDone = true;
    return err;
-}                               
+}
 
 /*
  * GotRequiredParams - check if all required parameters are retrieved
