@@ -20,9 +20,9 @@ Deliberate Syntax Error
 
 #if MACINTOSH
 #include <stdio.h>
-char *FileNameMacToUnix(const char *fn);
-char *FileNameUnixToMac(const char *fn);
-char *FileNameMacConvert(char *(*func)(char *),const char *fn);
+char *FileNameMacToUnix(char *fn);
+char *FileNameUnixToMac(char *fn);
+char *FileNameMacConvert(char *(*func)(char *),char *fn);
 #endif					/* MACINTOSH */
 
 #if MSDOS
@@ -729,7 +729,7 @@ Deliberate Syntax Error
 static char FileName_newfn[100];
 
 char *
-FileNameUnixToMac(const char *fn) {
+FileNameUnixToMac(char *fn) {
   char *q,*e,*r;
   int full;
   
@@ -761,7 +761,7 @@ FileNameUnixToMac(const char *fn) {
 }
 
 char *
-FileNameMacToUnix(const char *fn) {
+FileNameMacToUnix(char *fn) {
   char *q,*e,*r;
   
   if (strchr(fn,':') == NULL) return fn;
@@ -799,7 +799,7 @@ FileNameMacToUnix(const char *fn) {
  *  Example:  FileNameMacConvert(FileNameMacToUnix,":MyDir:MyFile")
  */
 char *
-FileNameMacConvert(char *(*func)(char *),const char *fn) {
+FileNameMacConvert(char *(*func)(char *),char *fn) {
    char *newfp, *newmem;
 
    newfp = (*func)(fn);
