@@ -1482,7 +1482,7 @@ struct node *n;
    if ((out_file = fopen(cname, "w")) == NULL)
       err2("cannot open output file ", cname);
    else
-      addrmlst(cname);
+      addrmlst(cname, out_file);
       
    prologue(); /* output standard comments and preprocessor directives */
 
