@@ -9,7 +9,7 @@ struct b_proc Bnoproc;
 #ifdef EventMon
 struct b_iproc mt_llist = {
    6, (sizeof(struct b_proc) - sizeof(struct descrip)), Ollist,
-   0, -1,  0, 0, 0, {sizeof( "[...]")-1, "[...]"}};
+   0, -1,  0, 0, {sizeof( "[...]")-1, "[...]"}};
 #endif					/* EventMon */
 
 
@@ -338,10 +338,6 @@ struct errtab errtab[] = {
    125, "list, record, or set expected",
    126, "list or record expected",
 
-#ifdef EventMon
-   127, "improper event monitoring setup",
-#endif					/* EventMon */
-
 #ifdef Graphics
    140, "window expected",
    141, "program terminated by window manager",
