@@ -5,6 +5,9 @@
  *  ishift, ixor, [keyword], [load], ord, name, runerr, seq, sort, sortf,
  *  type, variable
  */
+#if !COMPILER
+#include "../h/opdefs.h"
+#endif					/* !COMPILER */
 
 "args(p) - produce number of arguments for procedure p."
 
@@ -171,6 +174,11 @@ function{1} copy(x)
             }
       table: {
          body {
+#ifdef TableFix
+	    if (cptable(&x, &result, BlkLoc(x)->table.size) == Error)
+	       runerr(0);
+	    return result;
+#else					/* TableFix */
             register int i;
             register word slotnum;
             tended union block *src;
@@ -213,7 +221,9 @@ function{1} copy(x)
 
             if (TooSparse(dst))
                hshrink(dst);
+	    Desc_EVValD(dst, E_Tcreate, D_Table);
 	    return table(dst);
+#endif					/* TableFix */
             }
          }
 
@@ -252,6 +262,7 @@ function{1} copy(x)
             d2 = old_rec->fields;
             while (i--)
                *d1++ = *d2++;
+	    Desc_EVValD(new_rec, E_Rcreate, D_Record);
             return record(new_rec);
             }
          }
@@ -689,9 +700,6 @@ function{1,*} seq(from, by)
        * Suspending wipes out some things needed by the trace back code to
        *  render the offending expression. Restore them.
        */
-
-#include "../h/opdefs.h"
-
       lastop = Op_Invoke;
       xnargs = 2;
       xargp = r_args;
@@ -704,6 +712,45 @@ function{1,*} seq(from, by)
       }
 end
 
+"serial(x) - return serial number of structure."
+
+function {0,1} serial(x)
+   abstract {
+      return integer
+      }
+
+   type_case x of {
+      list:   inline {
+         return C_integer BlkLoc(x)->list.id;
+         }
+      set:   inline {
+         return C_integer BlkLoc(x)->set.id;
+         }
+      table:   inline {
+         return C_integer BlkLoc(x)->table.id;
+         }
+      record:   inline {
+         return C_integer BlkLoc(x)->record.id;
+         }
+      coexpr:   inline {
+         return C_integer BlkLoc(x)->coexpr.id;
+         }
+#ifdef Graphics
+      file:   inline {
+	 if (BlkLoc(x)->file.status & Fs_Window) {
+	    wsp ws = ((wbp)(BlkLoc(x)->file.fd))->window;
+	    return C_integer ws->serial;
+	    }
+	 else {
+	    fail;
+	    }
+         }
+#endif					/* Graphics */
+      default:
+         inline { fail; }
+      }
+end
+
 "sort(x,i) - sort structure x by method i (for tables)"
 
 function{1} sort(t, i)
@@ -750,6 +797,9 @@ function{1} sort(t, i)
             Protect(lp = alclist(size), runerr(0));
             Protect(ep = (union block *)alclstb(size,(word)0,size), runerr(0));
             lp->listhead = lp->listtail = ep;
+#ifdef ListFix
+            ep->lelem.listprev = ep->lelem.listnext = (union block *) lp;
+#endif					/* ListFix */
             bp = BlkLoc(t);  /* need not be tended if not set until now */
 
             if (size > 0) {  /* only need to sort non-empty records */
@@ -786,6 +836,9 @@ function{1} sort(t, i)
             Protect(lp = alclist(size), runerr(0));
             Protect(ep = (union block *)alclstb(size,(word)0,size), runerr(0));
             lp->listhead = lp->listtail = ep;
+#ifdef ListFix
+            ep->lelem.listprev = ep->lelem.listnext = (union block *)lp;
+#endif					/* ListFix */
             bp = BlkLoc(t);  /* need not be tended if not set until now */
 
             if (size > 0) {  /* only need to sort non-empty sets */
@@ -852,6 +905,9 @@ function{1} sort(t, i)
                Protect(lp = alclist(size), runerr(0));
                Protect(ep=(union block *)alclstb(size,(word)0,size),runerr(0));
                lp->listtail = lp->listhead = ep;
+#ifdef ListFix
+               ep->lelem.listprev = ep->lelem.listnext = (union block *) lp;
+#endif					/* ListFix */
                /*
                 * If the table is empty, there is no need to sort anything.
                 */
@@ -869,11 +925,21 @@ function{1} sort(t, i)
                n = 0;				/* list index */
                for (j = 0; j < HSegs && (seg = bp->hdir[j]) != NULL; j++)
                   for (k = segsize[j] - 1; k >= 0; k--)
-                     for (ep= seg->hslots[k]; ep != NULL; ep= ep->telem.clink){
+                     for (ep= seg->hslots[k];
+#ifdef TableFix
+			  BlkType(ep) == T_Telem;
+#else					/* TableFix */
+			  ep != NULL;
+#endif					/* TableFix */
+			  ep = ep->telem.clink){
                         Protect(tp = alclist((word)2), runerr(0));
                         Protect(ev = (union block *)alclstb((word)2,
                            (word)0, (word)2), runerr(0));
                         tp->listhead = tp->listtail = ev;
+#ifdef ListFix
+                        ev->lelem.listprev = ev->lelem.listnext =
+			   (union block *)tp;
+#endif					/* ListFix */
                         tp->listhead->lelem.lslots[0] = ep->telem.tref;
                         tp->listhead->lelem.lslots[1] = ep->telem.tval;
                         d1 = &lp->listhead->lelem.lslots[n++];
@@ -914,6 +980,9 @@ function{1} sort(t, i)
             Protect(lp = alclist(size), runerr(0));
             Protect(ep = (union block *)alclstb(size,(word)0,size), runerr(0));
             lp->listhead = lp->listtail = ep;
+#ifdef ListFix
+            ep->lelem.listprev = ep->lelem.listnext = (union block *)lp;
+#endif					/* ListFix */
             /*
              * If the table is empty there's no need to sort anything.
              */
@@ -935,7 +1004,13 @@ function{1} sort(t, i)
 
             for (j = 0; j < HSegs && (seg = bp->hdir[j]) != NULL; j++)
                for (k = segsize[j] - 1; k >= 0; k--)
-                  for (ep = seg->hslots[k]; ep != NULL; ep = ep->telem.clink) {
+                  for (ep = seg->hslots[k];
+#ifdef TableFix
+		       BlkType(ep) == T_Telem;
+#else					/* TableFix */
+		       ep != NULL;
+#endif					/* TableFix */
+		       ep = ep->telem.clink) {
                      *d1++ = ep->telem.tref;
                      *d1++ = ep->telem.tval;
                      }
@@ -1093,6 +1168,9 @@ function{1} sortf(t, i)
             Protect(lp = alclist(size), runerr(0));
             Protect(ep = (union block *)alclstb(size,(word)0,size), runerr(0));
             lp->listhead = lp->listtail = ep;
+#ifdef ListFix
+            ep->lelem.listprev = ep->lelem.listnext = (union block *) lp;
+#endif					/* ListFix */
             bp = BlkLoc(t);  /* need not be tended if not set until now */
 
             if (size > 0) {  /* only need to sort non-empty records */
@@ -1138,6 +1216,9 @@ function{1} sortf(t, i)
             Protect(lp = alclist(size), runerr(0));
             Protect(ep = (union block *)alclstb(size,(word)0,size), runerr(0));
             lp->listhead = lp->listtail = ep;
+#ifdef ListFix
+            ep->lelem.listprev = ep->lelem.listnext = (union block *)lp;
+#endif					/* ListFix */
             bp = BlkLoc(t);  /* need not be tended if not set until now */
 
             if (size > 0) {  /* only need to sort non-empty sets */
@@ -1371,10 +1452,13 @@ function{0,1} variable(s)
 end
 
 #ifdef MultiThread
-#include "../h/opdefs.h"
+
 "cofail(CE) - transmit a co-expression failure to CE"
 
-function{1} cofail(CE)
+function{0,1} cofail(CE)
+   abstract {
+      return any_value
+      }
    if is:null(CE) then
       body {
 	 struct b_coexpr *ce = topact((struct b_coexpr *)BlkLoc(k_current));
@@ -1393,6 +1477,214 @@ function{1} cofail(CE)
       }
 end
 
+
+"fieldnames(r) - generate the fieldnames of record r"
+
+function{*} fieldnames(r)
+   abstract {
+      return string
+      }
+   if !is:record(r) then runerr(107,r)
+   body {
+      int i;
+      for(i=0;i<BlkLoc(r)->record.recdesc->proc.nfields;i++) {
+	 suspend BlkLoc(r)->record.recdesc->proc.lnames[i];
+         }
+      fail;
+      }
+end
+
+
+"localnames(ce,i) - produce the names of local variables"
+" in the procedure activation i levels up in ce"
+function{*} localnames(ce,i)
+   declare {
+      tended struct descrip d;
+      }
+   abstract {
+      return string
+      }
+   if is:null(ce) then inline {
+      d = k_current;
+      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
+      }
+   else if is:proc(ce) then inline {
+      int j;
+      struct b_proc *cproc = (struct b_proc *)BlkLoc(ce);
+      for(j = 0; j < cproc->ndynam; j++) {
+	 result = cproc->lnames[j + cproc->nparam];
+	 suspend result;
+         }
+      fail;
+      }
+   else if is:coexpr(ce) then inline {
+      d = ce;
+      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
+      }
+   else runerr(118, ce)
+   if !def:C_integer(i,0) then
+      runerr(101,i)
+   body {
+#if !COMPILER
+      int j;
+      dptr arg;
+      struct b_proc *cproc;
+      struct pf_marker *thePfp = BlkLoc(d)->coexpr.es_pfp;
+
+      if (thePfp == NULL) fail;
+      
+      /*
+       * Produce error if i is negative
+       */
+      if (i < 0) {
+         irunerr(205, i);
+         errorfail;
+         }
+
+      while (i--) {
+	 thePfp = thePfp->pf_pfp;
+	 if (thePfp == NULL) fail;
+         }
+
+      arg = &((dptr)thePfp)[-(thePfp->pf_nargs) - 1];
+      cproc = (struct b_proc *)BlkLoc(arg[0]);    
+      for(j = 0; j < cproc->ndynam; j++) {
+	 result = cproc->lnames[j + cproc->nparam];
+	 suspend result;
+         }
+#endif					/* !COMPILER */
+      fail;
+      }
+end
+
+
+
+"staticnames(ce,i) - produce the names of static variables"
+" in the current procedure activation in ce"
+
+function{*} staticnames(ce,i)
+   declare {
+      tended struct descrip d;
+      }
+   abstract {
+      return string
+      }
+   if is:null(ce) then inline {
+      d = k_current;
+      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
+      }
+   else if is:proc(ce) then inline {
+      int j;
+      struct b_proc *cproc = (struct b_proc *)BlkLoc(ce);
+      for(j = 0; j < cproc->nstatic; j++) {
+	 result = cproc->lnames[j + cproc->nparam + cproc->ndynam];
+	 suspend result;
+         }
+      fail;
+      }
+   else if is:coexpr(ce) then inline {
+      d = ce;
+      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
+      }
+   else runerr(118,ce)
+   if !def:C_integer(i,0) then
+      runerr(101,i)
+   body {
+#if !COMPILER
+      int j;
+      dptr arg;
+      struct b_proc *cproc;
+      struct pf_marker *thePfp = BlkLoc(d)->coexpr.es_pfp;
+      if (thePfp == NULL) fail;
+
+      /*
+       * Produce error if i is negative
+       */
+      if (i < 0) {
+         irunerr(205, i);
+         errorfail;
+         }
+
+      while (i--) {
+	 thePfp = thePfp->pf_pfp;
+	 if (thePfp == NULL) fail;
+         }
+
+      arg = &((dptr)thePfp)[-(thePfp->pf_nargs) - 1];
+      cproc = (struct b_proc *)BlkLoc(arg[0]);    
+      for(j=0; j < cproc->nstatic; j++) {
+	 result = cproc->lnames[j + cproc->nparam + cproc->ndynam];
+	 suspend result;
+         }
+#endif					/* !COMPILER */
+      fail;
+      }
+end
+
+"paramnames(ce,i) - produce the names of the parameters"
+" in the current procedure activation in ce"
+
+function{1,*} paramnames(ce,i)
+   declare {
+      tended struct descrip d;
+      }
+   abstract {
+      return string
+      }
+   if is:null(ce) then inline {
+      d = k_main;
+      BlkLoc(k_main)->coexpr.es_pfp = pfp; /* sync w/ current value */
+      }
+   else if is:proc(ce) then inline {
+      int j;
+      struct b_proc *cproc = (struct b_proc *)BlkLoc(ce);
+      for(j = 0; j < cproc->nparam; j++) {
+	 result = cproc->lnames[j];
+	 suspend result;
+         }
+      fail;
+      }
+   else if is:coexpr(ce) then inline {
+      d = ce;
+      BlkLoc(k_main)->coexpr.es_pfp = pfp; /* sync w/ current value */
+      }
+   else runerr(118,ce)
+   if !def:C_integer(i,0) then
+      runerr(101,i)
+   body {
+#if !COMPILER
+      int j;
+      dptr arg;
+      struct b_proc *cproc;
+      struct pf_marker *thePfp = BlkLoc(d)->coexpr.es_pfp;
+
+      if (thePfp == NULL) fail;
+
+      /*
+       * Produce error if i is negative
+       */
+      if (i < 0) {
+         irunerr(205, i);
+         errorfail;
+         }
+
+      while (i--) {
+	 thePfp = thePfp->pf_pfp;
+	 if (thePfp == NULL) fail;
+         }
+
+      arg = &((dptr)thePfp)[-(thePfp->pf_nargs) - 1];
+      cproc = (struct b_proc *)BlkLoc(arg[0]);    
+      for(j = 0; j < cproc->nparam; j++) {
+	 result = cproc->lnames[j];
+	 suspend result;
+         }
+#endif					/* !COMPILER */
+      fail;
+      }
+end
+
+
 "load(s,arglist,input,output,error,blocksize,stringsize,stacksize) - load"
 " an icode file corresponding to string s as a co-expression."
 
@@ -1431,12 +1723,13 @@ function{1} load(s,arglist,infile,outfile,errfile,
        * Fragments of pseudo-icode to get loaded programs started,
        * and to handle termination.
        */
-      static word *lterm;
       static word pstart[7];
+      static word *lterm;
 
       inst tipc;
 
       tipc.opnd = pstart;
+      *tipc.op++ = Op_Noop; /* aligns Invokes operand */  /* ?cj? */
       *tipc.op++ = Op_Invoke;
       *tipc.opnd++ = 1;
       *tipc.op++ = Op_Coret;
@@ -1521,7 +1814,11 @@ function{1} load(s,arglist,infile,outfile,errfile,
        *  is transferred to lterm, the address of an ...
        */
       newefp = (struct ef_marker *)(sp+1);
-      newefp->ef_failure.opnd = lterm;
+#if IntBits != WordBits
+      newefp->ef_failure.op = (int *)lterm;
+#else					/* IntBits != WordBits */
+      newefp->ef_failure.op = lterm;
+#endif					/* IntBits != WordBits */
 
       newefp->ef_gfp = 0;
       newefp->ef_efp = 0;
@@ -1584,6 +1881,7 @@ function{1} parent(ce)
       }
 end
 
+#ifdef EventMon
 
 "eventmask(ce,cs) - given a ce, get or set that program's event mask"
 
@@ -1610,21 +1908,7 @@ function{1} eventmask(ce,cs)
          }
       }
 end
-
-
-"fieldnames(r) - generate the fieldnames of record r"
-
-function{*} fieldnames(r)
-   if !is:record(r) then runerr(107,r)
-   body {
-      int i;
-      for(i=0;i<BlkLoc(r)->record.recdesc->proc.nfields;i++) {
-	 suspend BlkLoc(r)->record.recdesc->proc.lnames[i];
-         }
-      fail;
-      }
-end
-
+#endif					/* EventMon */
 
 
 "globalnames(ce) - produce the names of identifiers global to ce"
@@ -1844,67 +2128,7 @@ function{*} keyword(keyname,ce)
       runerr(205, keyname);
       }
 end
-
-"localnames(ce,i) - produce the names of local variables"
-" in the procedure activation i levels up in ce"
-function{*} localnames(ce,i)
-   declare {
-      tended struct descrip d;
-      }
-   abstract {
-      return string
-      }
-   if is:null(ce) then inline {
-      d = k_current;
-      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
-      }
-   else if is:proc(ce) then inline {
-      int j;
-      struct b_proc *cproc = (struct b_proc *)BlkLoc(ce);
-      for(j = 0; j < cproc->ndynam; j++) {
-	 result = cproc->lnames[j + cproc->nparam];
-	 suspend result;
-         }
-      fail;
-      }
-   else if is:coexpr(ce) then inline {
-      d = ce;
-      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
-      }
-   else runerr(118, ce)
-   if !def:C_integer(i,0) then
-      runerr(101,i)
-   body {
-      int j;
-      dptr arg;
-      struct b_proc *cproc;
-      struct pf_marker *thePfp = BlkLoc(d)->coexpr.es_pfp;
-
-      if (thePfp == NULL) fail;
-      
-      /*
-       * Produce error if i is negative
-       */
-      if (i < 0) {
-         irunerr(205, i);
-         errorfail;
-         }
-
-      while (i--) {
-	 thePfp = thePfp->pf_pfp;
-	 if (thePfp == NULL) fail;
-         }
-
-      arg = &((dptr)thePfp)[-(thePfp->pf_nargs) - 1];
-      cproc = (struct b_proc *)BlkLoc(arg[0]);    
-      for(j = 0; j < cproc->ndynam; j++) {
-	 result = cproc->lnames[j + cproc->nparam];
-	 suspend result;
-         }
-      fail;
-      }
-end
-
+#ifdef EventMon
 
 "opmask(ce,cs) - get or set ce's program's opcode mask"
 
@@ -1931,125 +2155,70 @@ function{1} opmask(ce,cs)
          }
       }
 end
+#endif					/* EventMon */
 
-"staticnames(ce,i) - produce the names of static variables"
-" in the current procedure activation in ce"
 
-function{*} staticnames(ce,i)
-   declare {
-      tended struct descrip d;
-      }
+"structure(x) -- generate all structures allocated in program x"
+function {*} structure(x)
+
+   if !is:coexpr(x) then
+       runerr(118, x)
+
    abstract {
-      return string
+      return list ++ set ++ table ++ record
       }
-   if is:null(ce) then inline {
-      d = k_current;
-      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
-      }
-   else if is:proc(ce) then inline {
-      int j;
-      struct b_proc *cproc = (struct b_proc *)BlkLoc(ce);
-      for(j = 0; j < cproc->nstatic; j++) {
-	 result = cproc->lnames[j + cproc->nparam + cproc->ndynam];
-	 suspend result;
-         }
-      fail;
-      }
-   else if is:coexpr(ce) then inline {
-      d = ce;
-      BlkLoc(k_current)->coexpr.es_pfp = pfp; /* sync w/ current value */
-      }
-   else runerr(118,ce)
-   if !def:C_integer(i,0) then
-      runerr(101,i)
+
    body {
-      int j;
-      dptr arg;
-      struct b_proc *cproc;
-      struct pf_marker *thePfp = BlkLoc(d)->coexpr.es_pfp;
-      if (thePfp == NULL) fail;
+      tended char *bp;
+      char *free;
+      tended struct descrip descr;
+      word type;
+      struct region *theregion, *rp;
 
-      /*
-       * Produce error if i is negative
-       */
-      if (i < 0) {
-         irunerr(205, i);
-         errorfail;
-         }
-
-      while (i--) {
-	 thePfp = thePfp->pf_pfp;
-	 if (thePfp == NULL) fail;
-         }
-
-      arg = &((dptr)thePfp)[-(thePfp->pf_nargs) - 1];
-      cproc = (struct b_proc *)BlkLoc(arg[0]);    
-      for(j=0; j < cproc->nstatic; j++) {
-	 result = cproc->lnames[j + cproc->nparam + cproc->ndynam];
-	 suspend result;
-         }
+#ifdef MultiThread
+      theregion = ((struct b_coexpr *)BlkLoc(x))->program->blockregion;
+#else
+      theregion = curblock;
+#endif
+      for(rp = theregion; rp; rp = rp->next) {
+	 bp = rp->base;
+	 free = rp->free;
+	 while (bp < free) {
+	    type = BlkType(bp);
+	    switch (type) {
+            case T_List:
+            case T_Set:
+            case T_Table:
+            case T_Record: {
+               BlkLoc(descr) = (union block *)bp;
+               descr.dword = type | F_Ptr | D_Typecode;
+               suspend descr;
+               }
+	       }
+	    bp += BlkSize(bp);
+	    }
+	 }
+      for(rp = theregion->prev; rp; rp = rp->prev) {
+	 bp = rp->base;
+	 free = rp->free;
+	 while (bp < free) {
+	    type = BlkType(bp);
+	    switch (type) {
+            case T_List:
+            case T_Set:
+            case T_Table:
+            case T_Record: {
+               BlkLoc(descr) = (union block *)bp;
+               descr.dword = type | F_Ptr | D_Typecode;
+               suspend descr;
+               }
+	       }
+	    bp += BlkSize(bp);
+	    }
+	 }
       fail;
       }
 end
 
-"paramnames(ce,i) - produce the names of the parameters"
-" in the current procedure activation in ce"
 
-function{1,*} paramnames(ce,i)
-   declare {
-      tended struct descrip d;
-      }
-   abstract {
-      return string
-      }
-   if is:null(ce) then inline {
-      d = k_main;
-      BlkLoc(k_main)->coexpr.es_pfp = pfp; /* sync w/ current value */
-      }
-   else if is:proc(ce) then inline {
-      int j;
-      struct b_proc *cproc = (struct b_proc *)BlkLoc(ce);
-      for(j = 0; j < cproc->nparam; j++) {
-	 result = cproc->lnames[j];
-	 suspend result;
-         }
-      fail;
-      }
-   else if is:coexpr(ce) then inline {
-      d = ce;
-      BlkLoc(k_main)->coexpr.es_pfp = pfp; /* sync w/ current value */
-      }
-   else runerr(118,ce)
-   if !def:C_integer(i,0) then
-      runerr(101,i)
-   body {
-      int j;
-      dptr arg;
-      struct b_proc *cproc;
-      struct pf_marker *thePfp = BlkLoc(d)->coexpr.es_pfp;
-
-      if (thePfp == NULL) fail;
-
-      /*
-       * Produce error if i is negative
-       */
-      if (i < 0) {
-         irunerr(205, i);
-         errorfail;
-         }
-
-      while (i--) {
-	 thePfp = thePfp->pf_pfp;
-	 if (thePfp == NULL) fail;
-         }
-
-      arg = &((dptr)thePfp)[-(thePfp->pf_nargs) - 1];
-      cproc = (struct b_proc *)BlkLoc(arg[0]);    
-      for(j = 0; j < cproc->nparam; j++) {
-	 result = cproc->lnames[j];
-	 suspend result;
-         }
-      fail;
-      }
-end
 #endif					/* MultiThread */
