@@ -213,7 +213,11 @@ function{1} detab(s,i[n])
       if (is_expanded)
          return result;
       else {
-	 MMStr(DiffPtrs(StrLoc(result),strfree)); /* note the deallocation */
+	 long n = DiffPtrs(StrLoc(result),strfree); /* note deallocation */
+	 if (n < 0)
+	    EVVal(-n, E_StrDeAlc);
+	 else
+	    EVVal(n, E_String);
 	 strtotal += DiffPtrs(StrLoc(result),strfree);
          strfree = StrLoc(result);		/* reset the free pointer */
          return s;				/* return original string */
@@ -330,14 +334,23 @@ function{1} entab(s,i[n])
        *  original string (and reset strfree) to conserve memory.
        */
       if (inserted) {
+	 long n;
          StrLen(result) = DiffPtrs(out,StrLoc(result));
-	 MMStr(DiffPtrs(out,strfree));		/* note the deallocation */
+	 n = DiffPtrs(out,strfree);		/* note the deallocation */
+	 if (n < 0)
+	    EVVal(-n, E_StrDeAlc);
+	 else
+	    EVVal(n, E_String);
 	 strtotal += DiffPtrs(out,strfree);
          strfree = out;				/* give back unused space */
          return result;				/* return new string */
          }
       else {
-	 MMStr(DiffPtrs(StrLoc(result),strfree)); /* note the deallocation */
+	 long n = DiffPtrs(StrLoc(result),strfree); /* note the deallocation */
+	 if (n < 0)
+	    EVVal(-n, E_StrDeAlc);
+	 else
+	    EVVal(n, E_String);
 	 strtotal += DiffPtrs(StrLoc(result),strfree);
          strfree = StrLoc(result);		/* reset free pointer */
          return s;				/* return original string */
