@@ -91,7 +91,11 @@ struct descrip *res;
    if (bp->nused <= 0) {
       bp = (struct b_lelem *) bp->listnext;
       hp->listhead = (union block *) bp;
+#ifdef ListFix
+      bp->listprev = (union block *) hp;
+#else					/* ListFix */
       bp->listprev = NULL;
+#endif					/* ListFix */
       }
 
    /*
@@ -129,8 +133,8 @@ function{0,1} get_or_pop(x)
       }
 
    body {
-      EVValD(&x, E_Lpop);
-      if (!c_get((struct b_list *)BlkLoc(x),&result)) fail;
+      EVValD(&x, E_Lget);
+      if (!c_get((struct b_list *)BlkLoc(x), &result)) fail;
       return result;
       }
 end
@@ -311,6 +315,9 @@ function{1} list(n, x)
       Protect(hp = alclist(size), runerr(0));
       Protect(bp = alclstb(nslots, (word)0, size), runerr(0));
       hp->listhead = hp->listtail = (union block *) bp;
+#ifdef ListFix
+      bp->listprev = bp->listnext = (union block *) hp;
+#endif					/* ListFix */
 
       /*
        * Initialize each slot.
@@ -365,7 +372,7 @@ function{0,1} member(s, x)
             EVValD(&x, E_Tsub);
 
             hn = hash(&x);
-            memb(BlkLoc(s),&x,hn,&res);
+            memb(BlkLoc(s), &x, hn, &res);
             if (res == 1)
                return x;
             else
@@ -412,7 +419,11 @@ function{0,1} pull(x)
       if (bp->nused <= 0) {
          bp = (struct b_lelem *) bp->listprev;
          hp->listtail = (union block *) bp;
+#ifdef ListFix
+         bp->listnext = (union block *) hp;
+#else					/* ListFix */
          bp->listnext = NULL;
+#endif					/* ListFix */
          }
 
       /*
@@ -480,6 +491,9 @@ dptr val;
          }
 
       BlkLoc(*l)->list.listhead->lelem.listprev = (union block *) bp;
+#ifdef ListFix
+      bp->listprev = BlkLoc(*l);
+#endif					/* ListFix */
       bp->listnext = BlkLoc(*l)->list.listhead;
       BlkLoc(*l)->list.listhead = (union block *) bp;
       }
@@ -574,6 +588,9 @@ function{1} push(x, vals[n])
 	       }
 
 	    hp->listhead->lelem.listprev = (union block *) bp;
+#ifdef ListFix
+	    bp->listprev = (union block *) hp;
+#endif					/* ListFix */
 	    bp->listnext = hp->listhead;
 	    hp->listhead = (union block *) bp;
 	    }
@@ -604,7 +621,6 @@ function{1} push(x, vals[n])
       }
 end
 
-#ifdef Graphics
 /*
  * c_put - C-level, nontending list put function
  */
@@ -658,6 +674,9 @@ struct descrip *val;
       ((struct b_list *)BlkLoc(*l))->listtail->lelem.listnext =
 	(union block *) bp;
       bp->listprev = ((struct b_list *)BlkLoc(*l))->listtail;
+#ifdef ListFix
+      bp->listnext = BlkLoc(*l);
+#endif					/* ListFix */
       ((struct b_list *)BlkLoc(*l))->listtail = (union block *) bp;
       }
 
@@ -676,7 +695,6 @@ struct descrip *val;
    bp->nused++;
    ((struct b_list *)BlkLoc(*l))->size++;
 }
-#endif					/* Graphics */
 
 
 "put(L, x1, ..., xN) - put elements onto end of list L."
@@ -750,6 +768,9 @@ function{1} put(x, vals[n])
 
 	    hp->listtail->lelem.listnext = (union block *) bp;
 	    bp->listprev = hp->listtail;
+#ifdef ListFix
+	    bp->listnext = (union block *)hp;
+#endif					/* ListFix */
 	    hp->listtail = (union block *) bp;
 	    }
 
@@ -796,6 +817,7 @@ function{1} set(l)
             ps = hmake(T_Set, (word)0, (word)0);
             if (ps == NULL)
                runerr(0);
+            Desc_EVValD(ps, E_Screate, D_Set);
             return set(ps);
             }
          }
@@ -833,7 +855,13 @@ function{1} set(l)
 	     */
 	    Protect(ne = alcselem(&nulldesc, (uword)0), runerr(0));
 
-            for (pb = pb->list.listhead; pb != NULL; pb = pb->lelem.listnext) {
+            for (pb = pb->list.listhead;
+#ifdef ListFix
+		 BlkType(pb) == T_Lelem;
+#else					/* ListFix */
+		 pb != NULL;
+#endif					/* ListFix */
+		 pb = pb->lelem.listnext) {
                for (i = 0; i < pb->lelem.nused; i++) {
                   j = pb->lelem.first + i;
                   if (j >= pb->lelem.nslots)
