@@ -525,6 +525,9 @@ Deliberate Syntax Error
 	 Protect(hp = alclist(0), runerr(0));
 	 Protect(bp = alclstb(MinListSlots, (word)0, 0), runerr(0));
 	 hp->listhead = hp->listtail = (union block *) bp;
+#ifdef ListFix
+	 bp->listprev = bp->listnext = (union block *) hp;
+#endif					/* ListFix */
 
 	 /*
 	  * loop through attributes, checking validity
@@ -649,15 +652,13 @@ function{0,1} read(f)
        * from it, unless input redirected
        */
       if (fp == stdin
-#ifdef PresentationManager
            && !(ConsoleFlags & StdInRedirect)
-#endif					/* PresentationManager */
            ) {
         fp = OpenConsole();
         status = Fs_Window | Fs_Read | Fs_Write;
         }
-#endif					/* ConsoleWindow */
 
+#endif					/* ConsoleWindow */
       /*
        * Use getstrg to read a line from the file, failing if getstrg
        *  encounters end of file. [[ What about -2?]]
@@ -671,8 +672,10 @@ function{0,1} read(f)
 	    slen = wgetstrg(sbuf,MaxReadStr,fp);
 	    if (slen == -1)
 	       runerr(141);
-	    if (slen < -1)
+	    if (slen == -2)
 	       runerr(143);
+	    if (slen == -3)
+               fail;
 	    }
 	 else
 #endif					/* Graphics */
@@ -732,7 +735,7 @@ function{0,1} reads(f,i)
       }
 
    body {
-      long tally;
+      long tally, nbytes;
       int status;
       FILE *fp;
       tended struct descrip s;
@@ -759,9 +762,7 @@ function{0,1} reads(f,i)
        * from it, unless input redirected
        */
       if (fp == stdin
-#ifdef PresentationManager
           && !(ConsoleFlags & StdInRedirect)
-#endif					/* PresentationManager */
           ) {
         fp = OpenConsole();
         status = Fs_Read | Fs_Write | Fs_Window;
@@ -796,8 +797,12 @@ function{0,1} reads(f,i)
 	 /*
 	  * We may not have used the entire amount of storage we reserved.
 	  */
-	 MMStr(DiffPtrs(StrLoc(s) + i, strfree));
-	 strtotal += DiffPtrs(StrLoc(s) + i, strfree);
+	 nbytes = DiffPtrs(StrLoc(s) + i, strfree);
+	 if (nbytes < 0)
+	    EVVal(-nbytes, E_StrDeAlc);
+	 else
+	    EVVal(nbytes, E_String);
+	 strtotal += nbytes;
 	 strfree = StrLoc(s) + i;
 	 return s;
 	 }
@@ -810,8 +815,10 @@ function{0,1} reads(f,i)
 	 tally = wlongread(StrLoc(s),sizeof(char),i,fp);
 	 if (tally == -1)
 	    runerr(141);
-	 else if (tally < -1)
+	 else if (tally == -2)
 	    runerr(143);
+	 else if (tally == -3)
+            fail;
 	 }
       else
 #endif					/* Graphics */
@@ -823,8 +830,12 @@ function{0,1} reads(f,i)
       /*
        * We may not have used the entire amount of storage we reserved.
        */
-      MMStr(DiffPtrs(StrLoc(s) + tally, strfree));
-      strtotal += DiffPtrs(StrLoc(s) + tally, strfree);
+      nbytes = DiffPtrs(StrLoc(s) + tally, strfree);
+      if (nbytes < 0)
+         EVVal(-nbytes, E_StrDeAlc);
+      else
+         EVVal(nbytes, E_String);
+      strtotal += nbytes;
       strfree = StrLoc(s) + tally;
       return s;
       }
@@ -1162,21 +1173,21 @@ end
       if ((k_errout.status & Fs_Write) == 0)
 	 runerr(213);
       else {
-#ifndef PresentationManager
+#ifndef ConsoleWindow
 	 f = k_errout.fd;
-#else					/* PresentationManager */
+#else					/* ConsoleWindow */
          f = (ConsoleFlags & StdErrRedirect) ? k_errout.fd : OpenConsole();
-#endif					/* PresentationManager */
+#endif					/* ConsoleWindow */
 	 }
 #else					/* error_out */
       if ((k_output.status & Fs_Write) == 0)
 	 runerr(213);
       else {
-#ifndef PresentationManager
+#ifndef ConsoleWindow
 	 f = k_output.fd;
-#else					/* PresentationManager */
+#else					/* ConsoleWindow */
          f = (ConsoleFlags & StdOutRedirect) ? k_output.fd : OpenConsole();
-#endif					/* PresentationManager */
+#endif					/* ConsoleWindow */
 	 }
 #endif					/* error_out */
       }
@@ -1259,17 +1270,17 @@ function {1} name(x[nargs])
       FILE *f = NULL;
       word status =
 #if terminate
-#ifndef PresentationManager
+#ifndef ConsoleWindow
 	k_errout.status;
-#else					/* PresentationManager */
+#else					/* ConsoleWindow */
         (ConsoleFlags & StdErrRedirect) ? k_errout.status : Fs_Read | Fs_Write | Fs_Window;
-#endif					/* PresentationManager */
+#endif					/* ConsoleWindow */
 #else					/* terminate */
-#ifndef PresentationManager
+#ifndef ConsoleWindow
 	k_output.status;
-#else					/* PresentationManager */
+#else					/* ConsoleWindow */
         (ConsoleFlags & StdOutRedirect) ? k_output.status : Fs_Read | Fs_Write | Fs_Window;
-#endif					/* PresentationManager */
+#endif					/* ConsoleWindow */
 #endif					/* terminate */
 
 #ifdef BadCode
