@@ -362,8 +362,6 @@ function{0,1} Couple(w,w2)
       }
    body {
       tended struct descrip sbuf, sbuf2;
-      tended struct b_list *hp;
-      tended struct b_lelem *bp;
       wbp wb, wb_new;
       wsp ws;
 
@@ -1585,8 +1583,8 @@ function{3} Pixel(argv[argc])
       return integer ++ string
       }
    body {
+      struct imgmem imem;
       C_integer x, y, width, height;
-      int x2, y2, width2, height2;
       wbp w;
       int warg = 0, slen, r;
       tended struct descrip lastval;
@@ -1607,32 +1605,70 @@ function{3} Pixel(argv[argc])
 #define min(x,y) (((x)>(y))?(y):(x))
 #endif
 
-      x2 = max(x,0);
-      y2 = max(y,0);
-      width2 = min(width, (int)ws->width - x2);
-      height2 = min(height, (int)ws->height - y2);
+      imem.x = max(x,0);
+      imem.y = max(y,0);
+      imem.width = min(width, (int)ws->width - imem.x);
+      imem.height = min(height, (int)ws->height - imem.y);
 
-      if (getpixel_init(w, x2, y2, width2, height2) == Failed) fail;
+      if (getpixel_init(w, &imem) == Failed) fail;
 
       lastval = emptystr;
 
       for (j=y; j < y + height; j++) {
          for (i=x; i < x + width; i++) {
-            getpixel(w, i, j, &rv, strout);
+            getpixel(w, i, j, &rv, strout, &imem);
             slen = strlen(strout);
             if (rv >= 0) {
+	       int signal;
                if (slen != StrLen(lastval) ||
                      strncmp(strout, StrLoc(lastval), slen)) {
                   Protect((StrLoc(lastval) = alcstr(strout, slen)), runerr(0));
                   StrLen(lastval) = slen;
                   }
-               suspend lastval;
+#if COMPILER
+               suspend lastval;		/* memory leak on vanquish */
+#else					/* COMPILER */
+	       /*
+		* suspend, but free up imem if vanquished; RTL workaround
+		* Needs implementing under the compiler.
+		*/
+	       r_args[0] = lastval;
+#ifdef EventMon
+	       if ((signal = interp(G_Fsusp, r_args)) != A_Resume) {
+#else					/* EventMon */
+	       if ((signal = interp(G_Csusp, r_args)) != A_Resume) {
+#endif					/* EventMon */
+		  tend = r_tend.previous;
+		  getpixel_term(w, &imem);
+		  VanquishReturn(signal);
+		  }
+#endif					/* COMPILER */
                }
             else {
-               suspend C_integer rv;
+#if COMPILER
+               suspend C_integer rv;	/* memory leak on vanquish */
+#else					/* COMPILER */
+	       int signal;
+	       /*
+		* suspend, but free up imem if vanquished; RTL workaround
+		* Needs implementing under the compiler.
+		*/
+	       r_args[0].dword = D_Integer;
+	       r_args[0].vword.integr = rv;
+#ifdef EventMon
+	       if ((signal = interp(G_Fsusp, r_args)) != A_Resume) {
+#else					/* EventMon */
+	       if ((signal = interp(G_Csusp, r_args)) != A_Resume) {
+#endif					/* EventMon */
+		  tend = r_tend.previous;
+		  getpixel_term(w, &imem);
+		  VanquishReturn(signal);
+		  }
+#endif					/* COMPILER */
                }
             }
          }
+      getpixel_term(w, &imem);
       fail;
       }
       }
@@ -1727,7 +1763,8 @@ function{0,1} ReadImage(argv[argc])
 
       x += w->context->dx;
       y += w->context->dy;
-      mystrncpy(filename, tmp, MaxFileName);   /* copy to loc that won't move*/
+      strncpy(filename, tmp, MaxFileName);   /* copy to loc that won't move*/
+      filename[MaxFileName] = '\0';
 
       /*
        * First try to read as a GIF file.
@@ -1886,12 +1923,17 @@ function{*} WAttrib(argv[argc])
                /*
                 * Read/write the attribute
                 */
-               if ((pass == 1) && strnchr(StrLoc(sbuf), '=', StrLen(sbuf))) {
-                  /*
-                   * pass 1: perform attribute assignments
-                   */
-                  switch (wattrib(w, StrLoc(sbuf), StrLen(sbuf),
-				  &sbuf2, answer)) {
+               if (pass == 1) {
+		  char *tmp_s = StrLoc(sbuf);
+		  char *tmp_s2 = StrLoc(sbuf) + StrLen(sbuf);
+		  for ( ; tmp_s < tmp_s2; tmp_s++)
+		     if (*tmp_s == '=') break;
+		  if (tmp_s < tmp_s2) {
+		     /*
+		      * pass 1: perform attribute assignments
+		      */
+		     switch (wattrib(w, StrLoc(sbuf), StrLen(sbuf),
+				     &sbuf2, answer)) {
 		     case Failed:
 		        /*
 			 * Mark the attribute so we don't produce a result
@@ -1900,45 +1942,50 @@ function{*} WAttrib(argv[argc])
 		        continue;
 		     case Error: runerr(0, argv[n]);
 		     }
-		  if (StrLen(sbuf) > 4) {
-		     if (!strncmp(StrLoc(sbuf), "pos=", 4)) config |= 1;
-		     if (StrLen(sbuf) > 5) {
-			if (!strncmp(StrLoc(sbuf), "posx=", 5)) config |= 1;
-			if (!strncmp(StrLoc(sbuf), "posy=", 5)) config |= 1;
-			if (!strncmp(StrLoc(sbuf), "rows=", 5)) config |= 2;
-			if (!strncmp(StrLoc(sbuf), "size=", 5)) config |= 2;
-			if (StrLen(sbuf) > 6) {
-			   if (!strncmp(StrLoc(sbuf), "width=", 6))
-			      config |= 2;
-			   if (!strncmp(StrLoc(sbuf), "lines=", 6))
-			      config |= 2;
-			   if (StrLen(sbuf) > 7) {
-			      if (!strncmp(StrLoc(sbuf), "height=", 7))
+		     if (StrLen(sbuf) > 4) {
+			if (!strncmp(StrLoc(sbuf), "pos=", 4)) config |= 1;
+			if (StrLen(sbuf) > 5) {
+			   if (!strncmp(StrLoc(sbuf), "posx=", 5)) config |= 1;
+			   if (!strncmp(StrLoc(sbuf), "posy=", 5)) config |= 1;
+			   if (!strncmp(StrLoc(sbuf), "rows=", 5)) config |= 2;
+			   if (!strncmp(StrLoc(sbuf), "size=", 5)) config |= 2;
+			   if (StrLen(sbuf) > 6) {
+			      if (!strncmp(StrLoc(sbuf), "width=", 6))
 				 config |= 2;
-			      if (!strncmp(StrLoc(sbuf), "resize=", 7))
+			      if (!strncmp(StrLoc(sbuf), "lines=", 6))
 				 config |= 2;
-			      if (StrLen(sbuf) > 8) {
-				 if (!strncmp(StrLoc(sbuf), "columns=", 8))
+			      if (StrLen(sbuf) > 7) {
+				 if (!strncmp(StrLoc(sbuf), "height=", 7))
 				    config |= 2;
-				 if (StrLen(sbuf) > 9) {
-				    if (!strncmp(StrLoc(sbuf), "geometry=", 9))
-				       config |= 3;
+				 if (!strncmp(StrLoc(sbuf), "resize=", 7))
+				    config |= 2;
+				 if (StrLen(sbuf) > 8) {
+				    if (!strncmp(StrLoc(sbuf), "columns=", 8))
+				       config |= 2;
+				    if (StrLen(sbuf) > 9) {
+				       if (!strncmp(StrLoc(sbuf),
+						    "geometry=", 9))
+					  config |= 3;
+				       }
 				    }
 				 }
 			      }
 			   }
 			}
 		     }
-                  }
+		  }
 	       /*
 		* pass 2: perform attribute queries, suspend result(s)
 		*/
                else if (pass==2) {
-		  char *stmp;
+		  char *stmp, *stmp2;
 		  /*
 		   * Turn assignments into queries.
 		   */
-		  if (stmp = strnchr(StrLoc(sbuf),'=',StrLen(sbuf)))
+ 		  for( stmp = StrLoc(sbuf), 
+ 		      stmp2 = stmp + StrLen(sbuf); stmp < stmp2; stmp++)
+ 		     if (*stmp == '=') break;
+ 		  if (stmp < stmp2)
 		     StrLen(sbuf) = stmp - StrLoc(sbuf);
 
 		  switch (wattrib(w, StrLoc(sbuf), StrLen(sbuf),
