@@ -9,14 +9,29 @@
 #include "../h/header.h"
 #include "../h/opdefs.h"
 
+#if SCCX_MX
+/*
+ * The setsize program will replace "frog" with the file size of
+ * iconx.exe.  The file size is used to scan to the end of the file
+ * to see if a .icx file has been concatenated to the end.
+ */
+setint_t    sizevar = {'f','r','o','g'};
+int  thisIsIconx;
+char settingsname[260];
+
+/* This sets the stack size so it runs in a Windows DOS box */
+unsigned _stack = 100000;
+
+#endif  /* SCCX_MX */
+
 /*
  * Prototypes.
  */
-
 hidden	novalue	env_err	Params((char *msg,char *name,char *val));
 novalue	icon_setup	Params((int argc, char **argv, int *ip));
 
 #ifdef MacGraph
+void MacMain (int argc, char **argv);
 void ToolBoxInit (void);
 void MenuBarInit (void);
 void MouseInfoInit (void);
@@ -70,34 +85,136 @@ word istart[4];
 int mterm = Op_Quit;
 
 
+FILE *finredir, *fouredir, *ferredir;
 
 #ifdef MSWindows
+void detectRedirection()
+{
+   struct stat sb;
+   /*
+    * Look at the standard file handles and attempt to detect
+    * redirection.
+    */
+   if (fstat(stdin->_file, &sb) == 0) {
+      if (sb.st_mode & S_IFCHR) {		/* stdin is a device */
+	 }
+      if (sb.st_mode & S_IFREG) {		/* stdin is a regular file */
+	 }
+      /* stdin is of size sb.st_size */
+      if (sb.st_size > 0) {
+         ConsoleFlags |= StdInRedirect;
+	 }
+      }
+   else {					/* unable to identify stdin */
+      }
+
+   if (fstat(stdout->_file, &sb) == 0) {
+      if (sb.st_mode & S_IFCHR) {		/* stdout is a device */
+	 }
+      if (sb.st_mode & S_IFREG) {		/* stdout is a regular file */
+	 }
+      /* stdout is of size sb.st_size */
+      if (sb.st_size == 0)
+         ConsoleFlags |= StdOutRedirect;
+      }
+   else {					/* unable to identify stdout */
+     }
+}
+
+
+
 int CmdParamToArgv(char *s, char ***avp)
    {
+   char tmp[MaxPath], dir[MaxPath];
    char *t, *t2;
-   int rv=0;
+   int rv=0, i;
+   FILE *f;
+
    t = salloc(s);
    t2 = t;
-   while (*t2) {
-      while (*t2 && isspace(*t2)) t2++;
-      if (!*t2) break;
-      rv++;
-      while (*t2 && !isspace(*t2)) t2++;
-      }
-   rv++; /* make room for "iconx" at front */
-   *avp = (char **)alloc((rv + 1) * sizeof(char *));
-   rv = 0;
-   (*avp)[rv++] = salloc("iconx.exe");
-   t2 = t;
-   while (*t2) {
-      while (*t2 && isspace(*t2)) t2++;
-      if (!*t2) break;
-      (*avp)[rv++] = t2;
-      while (*t2 && !isspace(*t2)) t2++;
-      if (*t2) *t2++ = '\0';
-      }
+   *avp = malloc(2 * sizeof(char *));
+   (*avp)[rv++] = salloc("wiconx.exe");
    (*avp)[rv] = NULL;
-   return rv;  
+
+   detectRedirection();
+
+   while (*t2) {
+      while (*t2 && isspace(*t2)) t2++;
+      switch (*t2) {
+	 case '\0': return rv;
+	 case '<': case '>': {
+	    /*
+	     * perform file redirection; this is for Windows 3.1
+	     * and other situations where Wiconx is launched from
+	     * a shell that does not process < and > characters.
+	     */
+	    char c = *t2++, buf[128], *t3;
+	    FILE *f;
+	    while (*t2 && isspace(*t2)) t2++;
+	    t3 = buf;
+	    while (*t2 && !isspace(*t2)) *t3++ = *t2++;
+	    *t3 = '\0';
+	    if (c == '<')
+	       f = fopen(buf, "r");
+	    else
+	       f = fopen(buf, "w");
+	    if (f == NULL)
+	       syserr("unable to redirect i/o\n");
+	    if (c == '<') {
+	       finredir = f;
+	       ConsoleFlags |= StdInRedirect;
+	       }
+	    else {
+	       fouredir = f;
+	       ConsoleFlags |= StdOutRedirect;
+	       }
+	    break;
+	    }
+	 case '"': {
+	    char *t3 = ++t2;			/* skip " */
+            while (*t2 && (*t2 != '"')) t2++;
+            if (*t2)
+	       *t2++ = '\0';
+	    *avp = realloc(*avp, (rv + 2) * sizeof (char *));
+	    (*avp)[rv++] = salloc(t3);
+            (*avp)[rv] = NULL;
+	    break;
+	    }
+         default: {
+            FINDDATA_T fd;
+	    char *t3 = t2;
+            while (*t2 && !isspace(*t2)) t2++;
+	    if (*t2)
+	       *t2++ = '\0';
+            strcpy(tmp, t3);
+	    if (!FINDFIRST(tmp, &fd)) {
+	       *avp = realloc(*avp, (rv + 2) * sizeof (char *));
+	       (*avp)[rv++] = salloc(t3);
+               (*avp)[rv] = NULL;
+               }
+	    else {
+               int end;
+               strcpy(dir, t3);
+	       do {
+	          end = strlen(dir)-1;
+	          while (end >= 0 && dir[end] != '\\' && dir[end] != '/' &&
+			dir[end] != ':') {
+                     dir[end] = '\0';
+		     end--;
+	             }
+		  strcat(dir, FILENAME(&fd));
+	          *avp = realloc(*avp, (rv + 2) * sizeof (char *));
+	          (*avp)[rv++] = salloc(dir);
+                  (*avp)[rv] = NULL;
+	          } while (FINDNEXT(&fd));
+	       FINDCLOSE(&fd);
+	       }
+            break;
+	    }
+         }
+      }
+   free(t);
+   return rv;
    }
 
 novalue MSStartup(HINSTANCE hInstance, HINSTANCE hPrevInstance)
@@ -114,7 +231,7 @@ novalue MSStartup(HINSTANCE hInstance, HINSTANCE hPrevInstance)
       wc.cbWndExtra = 0;
       wc.hInstance  = hInstance;
       wc.hIcon      = NULL;
-      wc.hCursor    = LoadCursor(NULL, IDC_ARROW);
+      wc.hCursor    = NULL;
       wc.hbrBackground = GetStockObject(WHITE_BRUSH);
       wc.lpszMenuName = NULL;
       wc.lpszClassName = "iconx";
@@ -138,6 +255,9 @@ int_PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    MSStartup(hInstance, hPrevInstance);
    if (setjmp(mark_sj) == 0)
       iconx(argc,argv);
+   while (--argc>=0)
+      free(argv[argc]);
+   free(argv);
    wfreersc();
    return 0;
 }
@@ -168,6 +288,79 @@ char **argv;
    {
    int i, slen;
 
+#if SCCX_MX
+    int     ctrlbrk;
+    FILE*   selfPtr;
+    int     new_argc;
+    char    **new_argv;
+    struct FIND *p;
+
+    selfPtr = fopen( argv[0], "rb");
+    fseek( selfPtr, 0L, SEEK_END);
+    if( ftell( selfPtr) > sizevar.value)
+    {
+        strcpy( settingsname, argv[0]);
+        ++argc;
+        for( i=argc-1; i>0; --i)  argv[i] = argv[i-1];
+        thisIsIconx = 1;
+    }
+    else
+        thisIsIconx = 0;
+
+    fclose( selfPtr);
+
+    /*
+     *  Expand wildcard file names:
+     *      Replace command line arguments with the file names that match
+     *      wildcard specifications.
+     *      Leave arguments that don't match file names as they are so the
+     *      icon program can deal with them.
+     */
+    new_argc = 1;
+    new_argv = malloc( new_argc * sizeof(char *));
+    new_argv[0] = malloc( (strlen(argv[0])+1) * sizeof(char));
+    memcpy( new_argv[0], argv[0], strlen(argv[0])+1);
+    for( i=1; i<argc; ++i)
+    {
+        if( (p = findfirst( argv[i], 0)) != NULL)
+        {
+            while( p)
+            {
+                new_argc++;
+                new_argv = realloc( new_argv, new_argc * sizeof(char *));
+                new_argv[new_argc-1] = malloc( (strlen(p->name)+1) * sizeof(char));
+                memcpy( new_argv[new_argc-1], p->name, strlen(p->name)+1);
+                p = findnext();
+            }
+        }
+        else
+        {
+            new_argc++;
+            new_argv = realloc( new_argv, new_argc * sizeof(char *));
+            new_argv[new_argc-1] = malloc( (strlen(argv[i])+1) * sizeof(char));
+            memcpy( new_argv[new_argc-1], argv[i], strlen(argv[i])+1);
+        }
+    }
+
+    argv = new_argv;
+    argc = new_argc;
+
+#if 0
+    if( argc < 2)
+    {
+    syntax error!
+    The Symantec license agreement requires you to include a copyright
+    notice in your program.  This is a good place for it.
+
+        fprintf( stderr,
+            "\nCopyright (c) 1996, Your Name, Your City, Your State.\n");
+        exit(1);
+    }
+#endif
+    ctrlbrk = dos_get_ctrl_break();
+    dos_set_ctrl_break(1);   /* Ensure proper Ctrl-C operation */
+#endif  /* SCCX_MX */
+
 #if AMIGA
 #if AZTEC_C
    struct Process *FindTask();
@@ -279,7 +472,7 @@ char **argv;
       argv++;
       }
 
-   if (argc <= 1) 
+   if (argc <= 1)
       error(NULL, "no icode file specified");
    /*
     * Call icon_init with the name of the icode file to execute.	[[I?]]
@@ -381,6 +574,10 @@ char **argv;
    interp(0,(dptr)NULL);                        /*      [[I?]] */
 #endif					/* CoProcesses */
 
+#if SCCX_MX
+    dos_set_ctrl_break(ctrlbrk);   /* Restore original Ctrl-C operation */
+#endif
+
    c_exit(NormalExit);
 }
 
@@ -516,7 +713,7 @@ int *ip;
  * resolve - perform various fix-ups on the data read from the icode
  *  file.
  */
-#ifdef MultiThread			/* MultiThread */
+#ifdef MultiThread
 novalue resolve(pstate)
 struct progstate *pstate;
 #else					/* MultiThread */
@@ -529,6 +726,7 @@ novalue resolve()
    register dptr dp;
    extern Omkrec();
    extern int ftsize;
+   extern struct header hdr;
 
 #ifdef MultiThread
    register struct progstate *savedstate;
@@ -586,10 +784,6 @@ novalue resolve()
           */
          StrLoc(pp->pname) = strcons + (uword)StrLoc(pp->pname);
 
-#ifdef MultiThread
-         pp->program = curpstate;
-#endif					/* MultiThread */
-
          if (pp->ndynam == -2) {
             /*
              * This procedure is a record constructor.	Make its entry point
@@ -695,7 +889,7 @@ void MouseInfoInit (void)
 
 void ToolBoxInit (void)
 {
-   InitGraf (&thePort);
+   InitGraf (&qd.thePort);
    InitFonts ();
    InitWindows ();
    InitMenus ();
@@ -710,13 +904,13 @@ void MenuBarInit (void)
    MenuHandle     menu;
    OSErr          myErr;
    long           feature;
-   
+
    menuBar = GetNewMBar (kMenuBar);
    SetMenuBar (menuBar);
-   
+
    menu = GetMHandle (kAppleMenu);
    AddResMenu (menu, 'DRVR');
-   
+
    DrawMenuBar ();
 }
 
@@ -724,11 +918,11 @@ void EventLoop ( void )
 {
    EventRecord event, *eventPtr;
    char theChar;
-   
+
    gDone = false;
    while ( gDone == false )
    {
-      if ( WaitNextEvent ( everyEvent, &event, kSleep, nil ) ) 
+      if ( WaitNextEvent ( everyEvent, &event, kSleep, nil ) )
          DoEvent ( &event );
    }
 }
@@ -740,7 +934,7 @@ void main ()
    MenuBarInit ();
    MouseInfoInit ();
    cmlArgs = "";
-   
+
    EventLoop ();
 }
 #endif					/* MacGraph */
