@@ -337,7 +337,7 @@ dptr cargp;
        Testb((word)ToAscii(E_Line), curpstate->eventmask)
        )) {
 
-      if (InRange(code, ipc.opnd, records)) {
+      if (InRange(code, ipc.opnd, ecode)) {
 	uword ipc_offset = DiffPtrs((char *)ipc.opnd, (char *)code);
 	uword size;
 	word temp_no;
@@ -633,7 +633,6 @@ Deliberate Syntax Error
 
 	 case Op_Asgn:		/* e1 := e2 */
 	    Setup_Op(2);
-	    /* FJL DerefArg(2); */
 	    Call_Cond;
 
 	 case Op_Swap:		/* e1 :=: e2 */
@@ -644,13 +643,11 @@ Deliberate Syntax Error
 	 case Op_Subsc: 	/* e1[e2] */
 	    PushNull;
 	    Setup_Op(3);
-	    /* FJL DerefArg(2); */
 	    Call_Cond;
 				/* Generative binary operators */
 
 	 case Op_Rasgn: 	/* e1 <- e2 */
 	    Setup_Op(2);
-	    /* FJL DerefArg(2); */
 	    Call_Gen;
 
 	 case Op_Rswap: 	/* e1 <-> e2 */
@@ -663,8 +660,6 @@ Deliberate Syntax Error
 	 case Op_Sect:		/* e1[e2:e3] */
 	    PushNull;
 	    Setup_Op(4);
-	    /* FJL DerefArg(2); */
-	    /* FJL DerefArg(3); */
 	    Call_Cond;
 				/* Generative ternary operators */
 
@@ -775,7 +770,12 @@ Deliberate Syntax Error
                           fatalerr(301, NULL);
 #endif					/* MultiThread */
 
-                  for (bp = bp->list.listhead; bp != NULL;
+                  for (bp = bp->list.listhead;
+#ifdef ListFix
+		       BlkType(bp) == T_Lelem;
+#else					/* ListFix */
+		       bp != NULL;
+#endif					/* ListFix */
                      bp = bp->lelem.listnext) {
                         for (i = 0; i < bp->lelem.nused; i++) {
                            j = bp->lelem.first + i;
