@@ -50,7 +50,12 @@ operator{*} ! bang(underef x -> dx)
              * each one, suspend with a variable pointing to each
              * element contained in the block.
              */
-            for (ep = BlkLoc(dx)->list.listhead; ep != NULL;
+            for (ep = BlkLoc(dx)->list.listhead;
+#ifdef ListFix
+		 BlkType(ep) == T_Lelem;
+#else					/* ListFix */
+		 ep != NULL;
+#endif					/* ListFix */
                  ep = ep->lelem.listnext){
                for (i = 0; i < ep->lelem.nused; i++) {
                   j = ep->lelem.first + i;
@@ -171,6 +176,7 @@ operator{*} ! bang(underef x -> dx)
             return store[type(dx).set_elem]
             }
          inline {
+            EVValD(&dx, E_Sbang);
             /*
              *  This is similar to the method for tables except that a
              *  value is returned instead of a variable.
@@ -353,7 +359,11 @@ operator{0,1} ? random(underef x -> dx)
             while (i >= j + bp->lelem.nused) {
                j += bp->lelem.nused;
                bp = bp->lelem.listnext;
-               if (bp == NULL)
+#ifdef ListFix
+               if (BlkType(bp) == T_List)
+#else					/* ListFix */
+	       if (bp == NULL)
+#endif					/* ListFix */
                   syserr("list reference out of bounds in random");
                }
             /*
@@ -404,9 +414,14 @@ operator{0,1} ? random(underef x -> dx)
              */
             for (i = 0; i < HSegs && (seg = bp->table.hdir[i]) != NULL; i++)
                for (j = segsize[i] - 1; j >= 0; j--)
-                  for (ep = seg->hslots[j]; ep != NULL; ep = ep->telem.clink)
-                     if (--n <= 0)
-			{
+                  for (ep = seg->hslots[j];
+#ifdef TableFix
+		       BlkType(ep) == T_Telem;
+#else					/* TableFix */
+		       ep != NULL;
+#endif					/* TableFix */
+		       ep = ep->telem.clink)
+                     if (--n <= 0) {
 			Protect(tp = alctvtbl(&dx, &ep->telem.tref, ep->telem.hashnum), runerr(0));
 			return tvtbl(tp);
 			}
@@ -690,7 +705,12 @@ operator{0,1} [] subsc(underef x -> dx,y)
              */
             bp = lp->listhead;
             j = 1;
-            while (bp != NULL && i >= j + bp->lelem.nused) {
+	    /*
+	     * y is in range, so bp can never be null here. if it was, a memory
+	     * violation would occur in the code that follows, anyhow, so
+	     * exiting the loop on a NULL bp makes no sense.
+	     */
+            while (i >= j + bp->lelem.nused) {
                j += bp->lelem.nused;
                bp = bp->lelem.listnext;
                }
