@@ -18,10 +18,6 @@ word set_ser = 1;	/* serial numbers for sets */
 word table_ser = 1;	/* serial numbers for tables */
 #endif					/* MultiThread */
 
-/*
- * Note: function calls beginning with "MM" are just empty macros
- * unless EventMon is defined.
- */
 
 /*
  * AlcBlk - allocate a block.
@@ -31,7 +27,7 @@ word table_ser = 1;	/* serial numbers for tables */
    struct region *rp = NULL;
 
 #ifdef MultiThread
-   MMAlc((word)nbytes,t_code);
+   EVVal((word)nbytes, typech[t_code]);
 #endif					/* MultiThread */
 
    /*
@@ -44,7 +40,7 @@ word table_ser = 1;	/* serial numbers for tables */
     * If monitoring, show the allocation.
     */
 #ifndef MultiThread
-   MMAlc((word)nbytes,t_code);
+   EVVal((word)nbytes, typech[t_code]);
 #endif
 
    /*
@@ -244,7 +240,7 @@ Deliberate Syntax Error
          ep = (struct b_coexpr *)malloc((msize)stksize);
 #endif                                  /* ATTM32 */
       }
-      if (ep == NULL) 
+      if (ep == NULL)
          ReturnErrNum(305, NULL);
 
    alcnum++;		/* increment allocation count since last g.c. */
@@ -328,19 +324,17 @@ int tcode;
    register int i;
    register struct b_set *ps;
    register struct b_table *pt;
-   word serial;
 
    if (tcode == T_Table) {
-      serial = table_ser++;
       AlcFixBlk(pt, b_table, T_Table);
       ps = (struct b_set *)pt;
+      ps->id = table_ser++;
       }
    else {	/* tcode == T_Set */
-      serial = set_ser++;
       AlcFixBlk(ps, b_set, T_Set);
+      ps->id = set_ser++;
       }
    ps->size = 0;
-   ps->id = serial;
    ps->mask = 0;
    for (i = 0; i < HSegs; i++)
       ps->hdir[i] = NULL;
@@ -526,7 +520,7 @@ register word slen;
 #ifdef EventMon
    if (!noMTevents)
 #endif					/* EventMon */
-      MMStr(slen);
+      EVVal(slen, E_String);
    s = StrLoc(ts);
 #endif					/* MultiThread */
 
@@ -636,7 +630,7 @@ union block *bp;
       syserr ("deallocation botch");
    rp->free = (char *)bp;
    blktotal -= nbytes;
-   MMAlc(-nbytes, 0);
+   EVVal(nbytes, E_BlkDeAlc);
 }
 
 /*
@@ -674,7 +668,11 @@ word nbytes;
    if (DiffPtrs(curr->end, curr->free) >= nbytes)
       return curr->free;		/* quick return: current region is OK */
 
+#if SCCX_MX
+   if( (rp = findgap(curr, nbytes)) != 0) {    /* check all regions on chain */
+#else					/* SCCX_MX */
    if (rp = findgap(curr, nbytes)) {	/* check all regions on chain */
+#endif					/* SCCX_MX */
       *pcurr = rp;			/* switch regions */
       return rp->free;
       }
@@ -712,7 +710,11 @@ word nbytes;
    if (newsize < MinAbrSize)
       newsize = MinAbrSize;
 
+#if SCCX_MX
+   if( (rp = newregion(nbytes, newsize)) != 0) {
+#else					/* SCCX_MX */
    if (rp = newregion(nbytes, newsize)) {
+#endif					/* SCCX_MX */
       rp->prev = curr;
       rp->next = NULL;
       curr->next = rp;
@@ -746,7 +748,11 @@ word nbytes;
          if (DiffPtrs(rp->end,rp->free) >= want)
             return rp->free;
          }
+#if SCCX_MX
+   if( (rp = findgap(curr, nbytes)) != 0) {
+#else					/* SCCX_MX */
    if (rp = findgap(curr, nbytes)) {
+#endif					/* SCCX_MX */
       *pcurr = rp;
       return rp->free;
       }
