@@ -314,11 +314,20 @@ int get_name(dp1,dp0)
                   i = varptr - &blkptr->lelem.lslots[blkptr->lelem.first] + 1;
                   if (i < 1)
                      i += blkptr->lelem.nslots;
+#ifdef ListFix
+                  while (BlkType(blkptr->lelem.listprev) == T_Lelem) {
+#else					/* ListFix */
                   while (blkptr->lelem.listprev != NULL) {
+#endif					/* ListFix */
                      blkptr = blkptr->lelem.listprev;
                      i += blkptr->lelem.nused;
                      }
-                  sprintf(sbuf,"L[%ld]",i);
+#ifdef ListFix
+                  sprintf(sbuf,"list_%d[%ld]",
+			  blkptr->lelem.listprev->list.id, i);
+#else					/* ListFix */
+                  sprintf(sbuf,"L[%ld]", i);
+#endif					/* ListFix */
                   i = strlen(sbuf);
                   Protect(StrLoc(*dp0) = alcstr(sbuf,i), return Error);
                   StrLen(*dp0) = i;
@@ -327,8 +336,14 @@ int get_name(dp1,dp0)
                   i = varptr - blkptr->record.fields;
                   proc = &blkptr->record.recdesc->proc;
 
-                  sprintf(sbuf,"%s.%s",StrLoc(proc->recname),
-                      StrLoc(proc->lnames[i]));
+#ifdef TableFix
+                  sprintf(sbuf,"record %s_%d.%s", StrLoc(proc->recname),
+			  blkptr->record.id,
+			  StrLoc(proc->lnames[i]));
+#else
+                  sprintf(sbuf,"%s.%s", StrLoc(proc->recname),
+			  StrLoc(proc->lnames[i]));
+#endif
 
                   i = strlen(sbuf);
                   Protect(StrLoc(*dp0) = alcstr(sbuf,i), return Error);
@@ -446,23 +461,36 @@ static int keyref(bp, dp)
    dptr dp;
    {
    char *s, *s2;
+   char sbuf[100];			/* buffer; might be too small */
    int i, len;
 
    if (getimage(&(bp->telem.tref),dp) == Error)
       return Error;	
 
    /*
-    * Allocate space, and copy the image surrounded by "T[" and "]"
+    * Allocate space, and copy the image surrounded by "table_n[" and "]"
     */
-   Protect(s = alcstr(NULL, StrLen(*dp) + 3), return Error);
    s2 = StrLoc(*dp);
    len = StrLen(*dp);
+#ifdef TableFix
+   if (BlkType(bp) == T_Tvtbl)
+      bp = bp->tvtbl.clink;
+   else
+      while(BlkType(bp) == T_Telem)
+         bp = bp->telem.clink;
+   sprintf(sbuf, "table_%d[", bp->table.id);
+#else					/* TableFix */
+   strcpy(sbuf, "T[");
+#endif					/* TableFix */
+   { char * dest = sbuf + strlen(sbuf);
+   strncpy(dest, s2, len);
+   dest[len] = '\0';
+   }
+   strcat(sbuf, "]");
+   len = strlen(sbuf);
+   Protect(s = alcstr(sbuf, len), return Error);
    StrLoc(*dp) = s;
-   StrLen(*dp) = StrLen(*dp) + 3;
-   *s++ = 'T'; *s++ = '[';
-   for (i = 0; i < len; i++)
-      *s++ = *s2++;
-   *s++ = ']';
+   StrLen(*dp) = len;
    return Succeeded;
    }
 
@@ -548,7 +576,7 @@ int l;
 #else					/* MVS */
       fprintf(stderr, "%-13s: %4d  ",f, l);
    else
-      fprintf(stderr, "              :      ");
+      fprintf(stderr, "             :       ");
 #endif					/* MVS */
 
    }
@@ -746,9 +774,6 @@ oneop:
    fflush(stderr);
    }
 
-
-
-
 
 /*
  * ctrace - procedure named s is being called with nargs arguments, the first
