@@ -17,8 +17,6 @@ hidden novalue	listimage
 hidden novalue	printimage	Params((FILE *f,int c,int q));
 hidden char *	csname		Params((dptr dp));
 
-hidden novalue	showlevel	Params((int n));
-hidden novalue	showline	Params((char *f,int l));
 
 /* 
  * eq - compare two Icon strings for equality
@@ -1268,7 +1266,7 @@ word *ipc;
    static two = 2;	/* some compilers generate bad code for division
 			   by a constant that is a power of two ... */
 
-   if (!InRange(code,ipc,records))
+   if (!InRange(code,ipc,ecode))
       return 0;
    ipc_offset = DiffPtrs((char *)ipc,(char *)code);
    base = ilines;
@@ -1329,7 +1327,7 @@ word *ipc;
    extern word *records;
 #endif					/* MultiThread */
 
-   if (!InRange(code,ipc,records))
+   if (!InRange(code,ipc,ecode))
       return "?";
    ipc_offset = DiffPtrs((char *)ipc,(char *)code);
    for (p = efilenms - 1; p >= filenms; p--)
@@ -1477,6 +1475,7 @@ dptr dp1, dp2;
          }
 
       integer: {
+
 #ifdef LargeInts
          if (Type(source) == T_Lrgint) {
             word slen;
@@ -1502,6 +1501,7 @@ dptr dp1, dp2;
 #else					/* LargeInts */
          cnv:string(source, *dp2);
 #endif					/* LargeInts */
+
 	 }
 
       real: {
@@ -1562,6 +1562,7 @@ dptr dp1, dp2;
              *	file(nm) where nm is the argument originally given to
              *	open.
              */
+
 #ifdef Graphics
 	    if (BlkLoc(source)->file.status & Fs_Window) {
 	       s = ((wbp)(BlkLoc(source)->file.fd))->window->windowlabel;
@@ -1577,15 +1578,18 @@ dptr dp1, dp2;
 	       }
 	    else {
 #endif					/* Graphics */
+
                s = StrLoc(BlkLoc(source)->file.fname);
                len = StrLen(BlkLoc(source)->file.fname);
                Protect (reserve(Strings, (len << 2) + 12), return Error);
 	       Protect(t = alcstr("file(", (word)(5)), return Error);
 	       StrLoc(*dp2) = t;
 	       StrLen(*dp2) = 5;
+
 #ifdef Graphics
 	     }
 #endif					/* Graphics */
+
             while (len-- > 0)
                StrLen(*dp2) += doimage(*s++, '\0');
             Protect(alcstr(")", (word)(1)), return Error);
@@ -1976,6 +1980,9 @@ dptr rslt;
     *  for the list.
     */
    hp->listhead = hp->listtail = (union block *)bp;
+#ifdef ListFix
+   bp->listprev = bp->listnext = (union block *)hp;
+#endif					/* ListFix */
 
    /*
     * Copy the arguments into the list
@@ -2013,6 +2020,9 @@ dptr rslt;
     *  for the list.
     */
    hp->listhead = hp->listtail = (union block *)bp;
+#ifdef ListFix
+   bp->listprev = bp->listnext = (union block *)hp;
+#endif					/* ListFix */
 
    /*
     * Copy the arguments into the list
