@@ -28,19 +28,6 @@ int lastEvFWidth = 0, lastEvLeading = 0, lastEvAscent = 0;
 uword xmod_control, xmod_shift, xmod_meta;
 #endif					/* MultiThread */
 
-/*
- * A non-existent library function (exported to fsys.c)
- */
-char *strnchr(s,ci,len)
-char *s;
-int ci;
-long len;
-   {
-   register long i;
-   char c = (char)ci;
-   for(i=0;i<len;i++,s++) if (*s == c) return s;
-   return NULL;
-   }
 
 /*
  * subscript the already-processed-events "queue" to index i.
@@ -68,7 +55,7 @@ int i;
 /*
  * get event from window, assigning to &x, &y, and &interval
  *
- * returns 0 for success, -1 if window died, -2 for malformed queue
+ * returns 0 for success, -1 if window died or EOF, -2 for malformed queue
  */
 int wgetevent(w,res)
 wbp w;
@@ -80,6 +67,16 @@ dptr res;
    if (wstates != NULL && wstates->next != NULL		/* if multiple windows*/
    && (BlkLoc(w->window->listp)->list.size == 0)) {	/* & queue is empty */
       while (BlkLoc(w->window->listp)->list.size == 0) {
+#ifdef MSWindows
+	 if (ISCURSORON(w) && w->window->hasCaret == 0) {
+	    wsp ws = w->window;
+	    CreateCaret(ws->iconwin, NULL, FWIDTH(w), FHEIGHT(w));
+	    SetCaretBlinkTime(500);
+	    SetCaretPos(ws->x, ws->y - ASCENT(w));
+	    ShowCaret(ws->iconwin);
+	    ws->hasCaret = 1;
+	    }
+#endif					/* MSWindows */
 	 if (pollevent() < 0)				/* poll all windows */
 	    break;					/* break on error */
 #if UNIX || VMS || OS2_32
@@ -123,7 +120,7 @@ dptr res;
 /*
  * get event from window (drop mouse events), no echo
  *
- * returns 1 for success, -1 if window died, -2 for malformed queue
+ * return: 1 = success, -1 = window died, -2 = malformed queue, -3 = EOF
  */
 int wgetchne(w,res)
 wbp w;
@@ -135,15 +132,19 @@ dptr res;
       i = wgetevent(w,res);
       if (i != 0)
 	 return i;
-      if (is:string(*res))
+      if (is:string(*res)) {
+#ifdef MSWindows
+         if (*StrLoc(*res) == '\032') return -3; /* control-Z gives EOF */
+#endif					/* MSWindows */
          return 1;
+	 }
       }
    }
 
 /*
  * get event from window (drop mouse events), with echo
  *
- * returns 1 for success, -1 if window died, -2 for malformed queue
+ * returns 1 for success, -1 if window died, -2 for malformed queue, -3 for EOF
  */
 int wgetche(w,res)
 wbp w;
@@ -216,21 +217,11 @@ wsp getactivewindow()
       }
    }
 
-/*
- * strncpy with guaranteed NUL termination
- */
-novalue mystrncpy(dest,src,n)
-char *dest, *src;
-int n;
-   {
-   strncpy(dest,src,n);
-   dest[n] = '\0';
-   }
-
 /*
  * wlongread(s,elsize,nelem,f) -- read string from window for reads(w)
  *
  * returns length(>=0) for success, -1 if window died, -2 for malformed queue
+ *  -3 on EOF
  */
 int wlongread(s, elsize, nelem, f)
 char *s;
@@ -244,6 +235,8 @@ FILE *f;
 
    while (l < bytes) {
      c = wgetche((wbp)f, &foo);
+     if (c == -3 && l > 0)
+	return l;
      if (c < 0)
 	return c;
      c = *StrLoc(foo);
@@ -264,6 +257,7 @@ FILE *f;
  * wgetstrg(s,maxlen,f) -- get string from window for read(w) or !w
  *
  * returns length(>=0) for success, -1 if window died, -2 for malformed queue
+ *  -3 for EOF, -4 if length was limited by maxi
  */
 int wgetstrg(s, maxlen, f)
 char *s;
@@ -277,6 +271,8 @@ FILE *f;
 
    while (l < maxlen) {
       c = wgetche((wbp)f,&foo);
+      if (c == -3 && l > 0)
+	 return l;
       if (c < 0)
 	 return c;
       c = *StrLoc(foo);
@@ -293,7 +289,7 @@ FILE *f;
           break;
         }
       }
-   return l;
+   return -4;
    }
 
 
@@ -416,8 +412,20 @@ char *s;
    posx = atol(s2);
    if (*s2 == '-') s2++;
    while (isdigit(*s2)) s2++;
+   if (*s2 == '.') {
+      s2++;
+      while (isdigit(*s2)) s2++;
+      }
    if (*s2++ != ',') return Error;
+   if (!isdigit(*s2) && (*s2 != '-')) return Error;
    posy = atol(s2);
+   if (*s2 == '-') s2++;
+   while (isdigit(*s2)) s2++;
+   if (*s2 == '.') {
+      s2++;
+      while (isdigit(*s2)) s2++;
+      }
+   if (*s2) return Error;
    if (posx < 0) {
       if (posy < 0) sprintf(tmp,"%d%d",posx,posy);
       else sprintf(tmp,"%d+%d",posx,posy);
@@ -445,8 +453,19 @@ char *s;
    width = atol(s2);
    if (*s2 == '-') s2++;
    while (isdigit(*s2)) s2++;
+   if (*s2 == '.') {
+      s2++;
+      while (isdigit(*s2)) s2++;
+      }
    if (*s2++ != ',') return Error;
    height = atol(s2);
+   if (*s2 == '-') s2++;
+   while (isdigit(*s2)) s2++;
+   if (*s2 == '.') {
+      s2++;
+      while (isdigit(*s2)) s2++;
+      }
+   if (*s2) return Error;
    sprintf(tmp,"%dx%d",width,height);
    return setgeometry(w,tmp);
    }
@@ -560,7 +579,7 @@ char *buf;
 long *r, *g, *b;
    {
    int len, mul;
-   char *fmt, c, cbuffer[MAXCOLORNAME];
+   char *fmt, c;
    double dr, dg, db;
 
    *r = *g = *b = 0L;
@@ -639,7 +658,7 @@ char *buf;
 long *r, *g, *b;
    {
    int len, very;
-   char *p, *ebuf, cbuffer[MAXCOLORNAME];
+   char c, *p, *ebuf, cbuffer[MAXCOLORNAME];
    float lgt, sat, blend, bl2, m1, m2;
    float h1, l1, s1, h2, l2, s2, r2, g2, b2;
 
@@ -649,16 +668,21 @@ long *r, *g, *b;
    while (isspace(buf[len-1]))
       len--;				/* trim trailing spaces */
 
-   /* copy spec, replacing spaces and hyphens with NULs */
    if (len >= sizeof(cbuffer))
       return 0;				/* if too long for valid Icon spec */
-   strncpy(cbuffer, buf, len);
-   ebuf = cbuffer + len;
-   for (buf = cbuffer; buf < ebuf; buf++)
-      if (*buf == ' ' || *buf == '-')
-         *buf = '\0';
-   *ebuf = '\0';
+
+   /*
+    * copy spec, lowering case and replacing spaces and hyphens with NULs
+    */
+   for(p = cbuffer; c = *buf; p++, buf++) {
+      if (isupper(c)) *p = tolower(c);
+      else if (c == ' ' || c == '-') *p = '\0';
+      else *p = c;
+      }
+   *p = '\0';
+
    buf = cbuffer;
+   ebuf = buf + len;
 
    /* check for "very" */
    if (strcmp(buf, "very") == 0) {
@@ -2140,7 +2164,7 @@ char * abuf;
    {
    char val[128], *valptr;
    struct descrip d;
-   char *mid, c;
+   char *mid, *midend, c;
    int r, a;
    C_integer tmp;
    long lenattr, lenval;
@@ -2156,7 +2180,11 @@ char * abuf;
    if (pollevent() == -1)
       fatalerr(141,NULL);
 
-   if (mid = strnchr(s, '=', len)) {
+   midend = s + len;
+   for (mid = s; mid < midend; mid++)
+      if (*mid == '=') break;
+      
+   if (mid < midend) {
       /*
        * set an attribute
        */
@@ -2164,8 +2192,10 @@ char * abuf;
       lenval  = len - lenattr - 1;
       mid++;
 
-      mystrncpy(abuf, s, lenattr);
-      mystrncpy(val, mid, lenval);
+      strncpy(abuf, s, lenattr);
+      abuf[lenattr] = '\0';
+      strncpy(val, mid, lenval);
+      val[lenval] = '\0';
       StrLen(d) = strlen(val);
       StrLoc(d) = val;
 
@@ -2461,14 +2491,16 @@ char * abuf;
       default:
 	 ReturnErrNum(145, Error);
 	 }
-      mystrncpy(abuf, s, len);
+      strncpy(abuf, s, len);
+      abuf[len] = '\0';
       }
    else {
       int a;
       /*
        * get an attribute
        */
-      mystrncpy(abuf, s, len);
+      strncpy(abuf, s, len);
+      abuf[len] = '\0';
       switch (a=si_s2i(attribs, abuf)) {
       case A_VISUAL:
 	 if (getvisual(w, abuf) == Failed) return Failed;
@@ -2519,7 +2551,8 @@ char * abuf;
 	 MakeInt(XTOCOL(w,ws->width), answer);
 	 break;
       case A_POS: case A_POSX: case A_POSY:
-	 if (getpos(w) == Failed) return Failed;
+	 if (getpos(w) == Failed)
+	    return Failed;
 	 switch (a) {
 	 case A_POS:
 	    sprintf(abuf, "%d,%d", ws->posx, ws->posy);
@@ -2618,7 +2651,11 @@ char * abuf;
 	 break;
       case A_GEOMETRY:
 	 if (getpos(w) == Failed) return Failed;
+#if MACINTOSH
+         if (ws->theWindow)
+#else					/* MACINTOSH */
          if (ws->win)
+#endif					/* MACINTOSH */
            sprintf(abuf, "%dx%d+%d+%d",
 		   ws->width, ws->height, ws->posx, ws->posy);
          else
@@ -2636,7 +2673,7 @@ char * abuf;
       case A_ICONIMAGE:
 	 if (ICONFILENAME(w) != NULL)
 	    sprintf(abuf, "%s", ICONFILENAME(w));
-	 else return Failed;
+	 else *abuf = '\0';
 	 MakeStr(abuf, strlen(abuf), answer);
 	 break;
       case A_ICONLABEL:
@@ -3019,15 +3056,22 @@ FILE *OpenConsole()
          curblock = (struct region *)malloc(sizeof (struct region));
          curstring->size = MaxStrSpace;
          curblock->size  = MaxAbrSize;
+#ifdef MultiThread
+         initalloc(1000, &rootpstate);
+#else					/* MultiThread */
          initalloc(1000);
+#endif					/* MultiThread */
          }
 
       /*
        * allocate an empty event queue
        */
-      hp = alclist(0);
-      bp = alclstb(MinListSlots, (word)0, 0);
+      if ((hp = alclist(0)) == NULL) return NULL;
+      if ((bp = alclstb(MinListSlots, (word)0, 0)) == NULL) return NULL;
       hp->listhead = hp->listtail = (union block *)bp;
+#ifdef ListFix
+      bp->listprev = bp->listnext = (union block *)hp;
+#endif					/* ListFix */
 
       /*
        * build the attribute list
@@ -3045,7 +3089,8 @@ FILE *OpenConsole()
       StrLoc(attrs[3]) = "reverse=on";
       StrLen(attrs[3]) = strlen("reverse=on");
 
-      mystrncpy(ConsoleTitle, StrLoc(kywd_prog), StrLen(kywd_prog));
+      strncpy(ConsoleTitle, StrLoc(kywd_prog), StrLen(kywd_prog));
+      ConsoleTitle[StrLen(kywd_prog)] = '\0';
       strcat(ConsoleTitle, " - console");
       ConsoleBinding = wopen(ConsoleTitle, hp, attrs, 3, &eindx);
 #endif					/* BORLAND_286 */
@@ -3134,11 +3179,12 @@ int Consoleprintf(char *format, ...)
  */
 int Consoleputc(int c, FILE *f)
    {
+   wbp console;
    if (ConsoleFlags & OutputToBuf) {
-     *ConsoleStringBufPtr++ = c;
-     *ConsoleStringBufPtr = '\0';
-     return 1;
-     }
+      *ConsoleStringBufPtr++ = c;
+      *ConsoleStringBufPtr = '\0';
+      return 1;
+      }
    if ((f == stdout && !(ConsoleFlags & StdOutRedirect)) || 
        (f == stderr && !(ConsoleFlags & StdErrRedirect))) {
       if (flog) fputc(c, flog);
